#include "instr_logger.h"
#include "plugin.h"
#include "../pluginsdk/_plugins.h"
#include "../pluginsdk/_dbgfunctions.h"
#include "../pluginsdk/bridgemain.h"

// For SHGetFolderPathA
#include <Shlobj.h>
#include <Shlwapi.h>
#pragma comment(lib, "Shlwapi.lib")
#pragma comment(lib, "Shell32.lib")

// Add specific script API includes
#include "../pluginsdk/_scriptapi_register.h"
#include "../pluginsdk/_scriptapi_memory.h"
#include "../pluginsdk/_scriptapi_module.h"
#include <ios>
// We're not using these directly, so comment them out
// #include "../pluginsdk/_scriptapi_function.h"
// #include "../pluginsdk/_scriptapi_stack.h"
// #include "../pluginsdk/_scriptapi_debug.h"
// #include "../pluginsdk/_scriptapi_symbol.h"
// #include "../pluginsdk/_scriptapi_argument.h"

// Settings section name
#define SETTINGS_SECTION "InstrLogger"
#define CUSTOM_INI_FILENAME "InstrLoggerSettings.ini"

// Instruction type definitions
#define INST_TYPE_CALL 1
#define INST_TYPE_JMP 2
#define INST_TYPE_JCOND 3
#define INST_TYPE_RET 4
#define INST_TYPE_UNKNOWN 0

// Max buffer sizes
#define MAX_DISASM_BUFFER 256
#define MAX_LABEL_SIZE 256
#define MAX_MODULE_SIZE 256
// MAX_SETTING_SIZE is already defined in bridgemain.h

// Debug log file for tracking issues
static std::ofstream g_debugLogFile;

// Кэш для модулей по адресу для более быстрой работы
static std::unordered_map<duint, std::string> g_moduleCache;
static std::mutex g_moduleCache_mutex;

// Кэш для символов по адресу для более быстрой работы
static std::unordered_map<duint, std::string> g_symbolCache;
static std::mutex g_symbolCache_mutex;

// Global pointers to SDK functions
static DBGFUNCTIONS g_dbgFunctions;

// Global logger instance
InstrLogger *g_instrLogger = nullptr;

// Helper function to get plugin directory
static std::string GetPluginDirectory()
{
  char pluginPath[MAX_PATH] = "";
  DebugLog("[%s] Getting plugin directory...\n", PLUGIN_NAME);

  // Try different possible module names for the plugin
  HMODULE hModule = NULL;

  // Try x64 module names first
  if (NULL != (hModule = GetModuleHandleA("InstrLogger_x64.dp64")))
  {
    DebugLog("[%s] Found module handle for InstrLogger_x64.dp64\n", PLUGIN_NAME);
  }
  else if (NULL != (hModule = GetModuleHandleA("InstrLogger.dp64")))
  {
    DebugLog("[%s] Found module handle for InstrLogger.dp64\n", PLUGIN_NAME);
  }
  // Try x32 module names next
  else if (NULL != (hModule = GetModuleHandleA("InstrLogger_x32.dp32")))
  {
    DebugLog("[%s] Found module handle for InstrLogger_x32.dp32\n", PLUGIN_NAME);
  }
  else if (NULL != (hModule = GetModuleHandleA("InstrLogger.dp32")))
  {
    DebugLog("[%s] Found module handle for InstrLogger.dp32\n", PLUGIN_NAME);
  }
  // Try just the base DLL name
  else if (NULL != (hModule = GetModuleHandleA("InstrLogger")))
  {
    DebugLog("[%s] Found module handle for InstrLogger\n", PLUGIN_NAME);
  }

  // First try: Get directory from module handle
  if (hModule != NULL)
  {
    DWORD length = GetModuleFileNameA(hModule, pluginPath, MAX_PATH);
    if (length > 0 && length < MAX_PATH)
    {
      DebugLog("[%s] Got module path: %s\n", PLUGIN_NAME, pluginPath);

      // Use more reliable method to get directory
      char dirPath[MAX_PATH] = "";
      DWORD dirLength = GetFullPathNameA(pluginPath, MAX_PATH, dirPath, NULL);
      if (dirLength > 0 && dirLength < MAX_PATH)
      {
        // Find last backslash
        char *lastSlash = strrchr(dirPath, '\\');
        if (lastSlash)
        {
          *(lastSlash + 1) = 0; // Truncate to directory
          std::string directory = std::string(dirPath);
          DebugLog("[%s] Plugin directory (normalized): %s\n", PLUGIN_NAME, directory.c_str());

          // Verify directory exists and is accessible
          DWORD attrs = GetFileAttributesA(dirPath);
          if (attrs != INVALID_FILE_ATTRIBUTES && (attrs & FILE_ATTRIBUTE_DIRECTORY))
          {
            DebugLog("[%s] Plugin directory verified to exist\n", PLUGIN_NAME);
            return directory;
          }
          else
          {
            DWORD error = GetLastError();
            DebugLog("[%s] Plugin directory verification failed, error: %lu\n", PLUGIN_NAME, error);
          }
        }
        else
        {
          DebugLog("[%s] Failed to find directory separator in path: %s\n", PLUGIN_NAME, dirPath);
        }
      }
      else
      {
        DWORD error = GetLastError();
        DebugLog("[%s] GetFullPathNameA failed, error: %lu\n", PLUGIN_NAME, error);
      }
    }
    else
    {
      DWORD error = GetLastError();
      DebugLog("[%s] Failed to get module filename, error: %lu\n", PLUGIN_NAME, error);
    }
  }
  else
  {
    DebugLog("[%s] Failed to get module handle for any plugin DLL name variation\n", PLUGIN_NAME);
  }

  // Second try: Get path from x64dbg executable and add plugins directory
  ZeroMemory(pluginPath, MAX_PATH);
  DWORD length = GetModuleFileNameA(NULL, pluginPath, MAX_PATH);
  if (length > 0 && length < MAX_PATH)
  {
    DebugLog("[%s] Got x64dbg executable path: %s\n", PLUGIN_NAME, pluginPath);

    // Use GetFullPathName to normalize path
    char dirPath[MAX_PATH] = "";
    DWORD dirLength = GetFullPathNameA(pluginPath, MAX_PATH, dirPath, NULL);
    if (dirLength > 0 && dirLength < MAX_PATH)
    {
      char *lastSlash = strrchr(dirPath, '\\');
      if (lastSlash)
      {
        *(lastSlash + 1) = 0; // Truncate to directory
        std::string directory = std::string(dirPath) + "plugins\\";
        DebugLog("[%s] Using plugins directory as fallback: %s\n", PLUGIN_NAME, directory.c_str());

        // Verify directory exists
        DWORD attrs = GetFileAttributesA(directory.c_str());
        if (attrs != INVALID_FILE_ATTRIBUTES && (attrs & FILE_ATTRIBUTE_DIRECTORY))
        {
          DebugLog("[%s] Plugins directory verified to exist\n", PLUGIN_NAME);
          return directory;
        }
        else
        {
          // Try to create plugins directory if it doesn't exist
          DWORD error = GetLastError();
          DebugLog("[%s] Plugins directory doesn't exist or can't be accessed, error: %lu\n", PLUGIN_NAME, error);

          // Try to create plugins directory
          if (CreateDirectoryA(directory.c_str(), NULL) || GetLastError() == ERROR_ALREADY_EXISTS)
          {
            DebugLog("[%s] Created or confirmed plugins directory\n", PLUGIN_NAME);
            return directory;
          }
          else
          {
            error = GetLastError();
            DebugLog("[%s] Failed to create plugins directory, error: %lu\n", PLUGIN_NAME, error);
          }
        }
      }
    }
  }

  // Third try: Check if we have a writable user profile directory
  char appDataPath[MAX_PATH] = "";
  if (SUCCEEDED(SHGetFolderPathA(NULL, CSIDL_LOCAL_APPDATA, NULL, 0, appDataPath)))
  {
    std::string directory = std::string(appDataPath) + "\\x64dbg\\plugins\\";
    DebugLog("[%s] Using AppData directory as fallback: %s\n", PLUGIN_NAME, directory.c_str());

    // Create directories if they don't exist
    if (CreateDirectoryA(directory.c_str(), NULL) || GetLastError() == ERROR_ALREADY_EXISTS)
    {
      DebugLog("[%s] AppData plugin directory is available\n", PLUGIN_NAME);
      return directory;
    }
    else
    {
      DWORD error = GetLastError();
      DebugLog("[%s] Failed to create AppData plugin directory, error: %lu\n", PLUGIN_NAME, error);
    }
  }

  // Final fallback: use current directory
  ZeroMemory(pluginPath, MAX_PATH);
  DWORD result = GetCurrentDirectoryA(MAX_PATH, pluginPath);
  if (result > 0 && result < MAX_PATH)
  {
    std::string directory = std::string(pluginPath);
    if (directory.back() != '\\')
      directory += '\\';
    DebugLog("[%s] Using current directory as final fallback: %s\n", PLUGIN_NAME, directory.c_str());

    // Verify directory is writable
    std::string testFile = directory + "test_write.tmp";
    HANDLE hTest = CreateFileA(testFile.c_str(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
                               FILE_ATTRIBUTE_NORMAL, NULL);
    if (hTest != INVALID_HANDLE_VALUE)
    {
      CloseHandle(hTest);
      DeleteFileA(testFile.c_str());
      DebugLog("[%s] Current directory is writable\n", PLUGIN_NAME);
      return directory;
    }
    else
    {
      DWORD error = GetLastError();
      DebugLog("[%s] Current directory is not writable, error: %lu\n", PLUGIN_NAME, error);
    }
  }

  // Absolute last resort: try to use temp directory
  char tempPath[MAX_PATH] = "";
  DWORD tempLen = GetTempPathA(MAX_PATH, tempPath);
  if (tempLen > 0 && tempLen < MAX_PATH)
  {
    std::string directory = std::string(tempPath) + "x64dbg_plugins\\";
    DebugLog("[%s] Using temp directory as absolute last resort: %s\n", PLUGIN_NAME, directory.c_str());

    // Create the directory if it doesn't exist
    if (CreateDirectoryA(directory.c_str(), NULL) || GetLastError() == ERROR_ALREADY_EXISTS)
    {
      DebugLog("[%s] Temp plugin directory is available\n", PLUGIN_NAME);
      return directory;
    }
  }

  DebugLog("[%s] All attempts to get plugin directory failed\n", PLUGIN_NAME);
  return ""; // Return empty string if all methods failed
}

// Function to save setting to custom INI file
static bool SaveSettingToIni(const char *section, const char *key, const char *value)
{
  std::string pluginDir = GetPluginDirectory();
  if (pluginDir.empty())
  {
    DebugLog("[%s] Failed to get plugin directory for INI file\n", PLUGIN_NAME);
    return false;
  }

  std::string iniPath = pluginDir + CUSTOM_INI_FILENAME;

  // Add detailed logging for debugging
  DebugLog("[%s] Saving setting [%s]:[%s]=%s to %s\n", PLUGIN_NAME, section, key, value, iniPath.c_str());

  // Force the directory to exist
  std::string dirPath = pluginDir;
  if (dirPath.back() == '\\')
  {
    dirPath.pop_back(); // Remove trailing backslash if present
  }

  DWORD dirAttrs = GetFileAttributesA(dirPath.c_str());
  if (dirAttrs == INVALID_FILE_ATTRIBUTES || !(dirAttrs & FILE_ATTRIBUTE_DIRECTORY))
  {
    DebugLog("[%s] Plugin directory may not exist, trying to create it: %s\n", PLUGIN_NAME, dirPath.c_str());
    if (!CreateDirectoryA(dirPath.c_str(), NULL) && GetLastError() != ERROR_ALREADY_EXISTS)
    {
      DWORD error = GetLastError();
      DebugLog("[%s] Failed to create directory, error: %lu\n", PLUGIN_NAME, error);
      return false;
    }
  }

  // Try to open the file directly to make sure we have access
  HANDLE hFile = CreateFileA(iniPath.c_str(), GENERIC_READ | GENERIC_WRITE, 0, NULL,
                             OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
  if (hFile == INVALID_HANDLE_VALUE)
  {
    DWORD error = GetLastError();
    DebugLog("[%s] Cannot access INI file directly, error: %lu\n", PLUGIN_NAME, error);
    return false;
  }
  CloseHandle(hFile);

  // Now write to the INI file
  BOOL result = WritePrivateProfileStringA(section, key, value, iniPath.c_str());
  if (result == 0)
  {
    DWORD error = GetLastError();
    DebugLog("[%s] WritePrivateProfileStringA failed with error %lu\n", PLUGIN_NAME, error);
    return false;
  }

  // Try to verify the write was successful
  char buffer[MAX_PATH] = "";
  DWORD readResult = GetPrivateProfileStringA(section, key, "", buffer, sizeof(buffer), iniPath.c_str());
  if (readResult == 0 && GetLastError() != ERROR_SUCCESS)
  {
    DWORD error = GetLastError();
    DebugLog("[%s] Failed to verify write - GetPrivateProfileStringA error %lu\n", PLUGIN_NAME, error);
    return false;
  }

  if (strcmp(buffer, value) != 0)
  {
    DebugLog("[%s] Verification failed! Wrote '%s' but read back '%s'\n", PLUGIN_NAME, value, buffer);
    return false;
  }

  // Flush system cache to make sure it's written
  WritePrivateProfileStringA(NULL, NULL, NULL, iniPath.c_str());
  DebugLog("[%s] Successfully wrote and verified setting [%s]:[%s]=%s\n", PLUGIN_NAME, section, key, value);
  return true;
}

// Function to load setting from custom INI file
static bool LoadSettingFromIni(const char *section, const char *key, char *buffer, DWORD size, const char *defaultValue = "")
{
  std::string pluginDir = GetPluginDirectory();
  if (pluginDir.empty())
  {
    DebugLog("[%s] Failed to get plugin directory for INI file\n", PLUGIN_NAME);
    if (defaultValue && defaultValue[0] != '\0')
    {
      strncpy_s(buffer, size, defaultValue, _TRUNCATE);
      DebugLog("[%s] Using default value '%s' for [%s]:%s\n", PLUGIN_NAME, defaultValue, section, key);
      return true;
    }
    return false;
  }

  std::string iniPath = pluginDir + CUSTOM_INI_FILENAME;

  // Log the attempt
  DebugLog("[%s] Reading setting [%s]:%s from %s\n", PLUGIN_NAME, section, key, iniPath.c_str());

  // Check if file exists first
  DWORD fileAttrs = GetFileAttributesA(iniPath.c_str());
  if (fileAttrs == INVALID_FILE_ATTRIBUTES)
  {
    DWORD error = GetLastError();
    DebugLog("[%s] INI file does not exist (error: %lu), using default value\n", PLUGIN_NAME, error);

    if (defaultValue && defaultValue[0] != '\0')
    {
      strncpy_s(buffer, size, defaultValue, _TRUNCATE);
      return true;
    }
    return false;
  }

  // Try to open the file directly to check access
  HANDLE hFile = CreateFileA(iniPath.c_str(), GENERIC_READ, FILE_SHARE_READ, NULL,
                             OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if (hFile == INVALID_HANDLE_VALUE)
  {
    DWORD error = GetLastError();
    DebugLog("[%s] Cannot open INI file for reading, error: %lu\n", PLUGIN_NAME, error);

    if (defaultValue && defaultValue[0] != '\0')
    {
      strncpy_s(buffer, size, defaultValue, _TRUNCATE);
      return true;
    }
    return false;
  }
  CloseHandle(hFile);

  // Read from the INI file
  DWORD result = GetPrivateProfileStringA(section, key, defaultValue, buffer, size, iniPath.c_str());

  if (result == 0 && GetLastError() != ERROR_SUCCESS)
  {
    DWORD error = GetLastError();
    DebugLog("[%s] GetPrivateProfileStringA failed with error %lu\n", PLUGIN_NAME, error);

    if (defaultValue && defaultValue[0] != '\0')
    {
      strncpy_s(buffer, size, defaultValue, _TRUNCATE);
      DebugLog("[%s] Using default value '%s' for [%s]:%s after error\n", PLUGIN_NAME, defaultValue, section, key);
      return true;
    }
    return false;
  }

  bool valueFound = result > 0;
  if (valueFound)
  {
    DebugLog("[%s] Found value '%s' for [%s]:%s\n", PLUGIN_NAME, buffer, section, key);
  }
  else if (defaultValue && defaultValue[0] != '\0')
  {
    DebugLog("[%s] Key not found, using default value '%s' for [%s]:%s\n", PLUGIN_NAME, defaultValue, section, key);
    valueFound = true;
  }
  else
  {
    DebugLog("[%s] Key not found and no default provided for [%s]:%s\n", PLUGIN_NAME, section, key);
  }

  return valueFound;
}

// Custom log function that writes to both x64dbg log and our debug log file
void DebugLog(const char *format, ...)
{
  char buffer[4096];
  va_list args;
  va_start(args, format);
  vsnprintf(buffer, sizeof(buffer), format, args);
  va_end(args);

  // Log to x64dbg
  _plugin_logprintf("%s", buffer);

  // Log to file if open
  if (g_debugLogFile.is_open())
  {
    // Get timestamp
    SYSTEMTIME st;
    GetLocalTime(&st);
    char timestamp[24];
    sprintf_s(timestamp, sizeof(timestamp), "%02d:%02d:%02d.%03d",
              st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);

    g_debugLogFile << timestamp << " | " << buffer;
    g_debugLogFile.flush(); // Make sure it's written immediately
  }
}

// Helper function to initialize debug log
static void InitDebugLog()
{
  if (g_debugLogFile.is_open())
    return;

  // Try to create debug log in plugin directory
  char pluginPath[MAX_PATH] = "";
  if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
      GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
  {

    char *lastSlash = strrchr(pluginPath, '\\');
    if (lastSlash)
    {
      *(lastSlash + 1) = 0; // Truncate to directory
      std::string debugLogPath = std::string(pluginPath) + "instrlogger_debug.log";
      g_debugLogFile.open(debugLogPath, std::ios::out | std::ios::trunc);

      if (g_debugLogFile.is_open())
      {
        DebugLog("[InstrLogger] Debug log initialized at: %s\n", debugLogPath.c_str());
        return;
      }
    }
  }

  // Fallback to current directory
  g_debugLogFile.open("instrlogger_debug.log", std::ios::out | std::ios::trunc);
  if (g_debugLogFile.is_open())
  {
    DebugLog("[InstrLogger] Debug log initialized in current directory\n");
  }
  else
  {
    _plugin_logprintf("[InstrLogger] WARNING: Failed to create debug log file!\n");
  }
}

// Helper function to get current module name
static std::string GetModuleNameFromAddr(duint addr)
{
  // Validate address
  if (!addr || !DbgMemIsValidReadPtr(addr))
    return "";

  // Check if we already have this address in cache
  {
    std::lock_guard<std::mutex> lock(g_moduleCache_mutex);
    auto it = g_moduleCache.find(addr);
    if (it != g_moduleCache.end())
      return it->second;
  }

  char moduleName[MAX_MODULE_SIZE] = "";
  bool success = false;

  try
  {
    // Try to get the module name at this address
    success = DbgGetModuleAt(addr, moduleName);
  }
  catch (...)
  {
    DebugLog("[InstrLogger] GetModuleNameFromAddr: exception in DbgGetModuleAt for address %p\n", (void *)addr);
    return "";
  }

  std::string result = "";
  if (success && moduleName[0] != '\0')
  {
    result = std::string(moduleName);

    // Store in cache for future use
    std::lock_guard<std::mutex> lock(g_moduleCache_mutex);
    g_moduleCache[addr] = result;
  }

  return result;
}

// Helper function to get current module's base and size
static bool GetCurrentModuleRange(duint &moduleBase, duint &moduleSize)
{
  _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: starting\n");

  // Initialize out parameters
  moduleBase = 0;
  moduleSize = 0;

  // Get current instruction pointer
  duint cip = 0;
  try
  {
    cip = Script::Register::GetCIP();
    if (!cip)
    {
      _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: CIP is null\n");

      // Simple fallback - we can't get a valid instruction pointer
      return false;
    }
    _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: CIP = %p\n", (void *)cip);
  }
  catch (...)
  {
    _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Exception getting CIP\n");
    return false;
  }

  // Get module at CIP
  char moduleName[MAX_MODULE_SIZE] = "";
  bool foundModule = false;

  try
  {
    foundModule = DbgGetModuleAt(cip, moduleName);
  }
  catch (...)
  {
    _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Exception in DbgGetModuleAt\n");
    return false;
  }

  if (!foundModule)
  {
    _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: No module at CIP\n");
    return false;
  }

  _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Found module: %s\n", moduleName);

  // Get module base address
  try
  {
    moduleBase = DbgModBaseFromName(moduleName);
  }
  catch (...)
  {
    _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Exception in DbgModBaseFromName\n");
    return false;
  }

  if (!moduleBase)
  {
    _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Failed to get module base\n");
    return false;
  }

  _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Module base: %p\n", (void *)moduleBase);

  // Get module size using Memory API
  try
  {
    if (!Script::Memory::GetSize(moduleBase, &moduleSize) || moduleSize == 0)
    {
      _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Failed to get module size\n");

      // Simple fallback - use a reasonable default size
      moduleSize = 0x1000000; // 16 MB - reasonable default for most modules
      _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Using default size: %p\n", (void *)moduleSize);
    }
  }
  catch (...)
  {
    _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Exception in Script::Memory::GetSize\n");
    // Default fallback size
    moduleSize = 0x1000000; // 16 MB
    _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Using default size after exception: %p\n", (void *)moduleSize);
  }

  _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Module size: %p\n", (void *)moduleSize);
  _plugin_logprintf("[InstrLogger] GetCurrentModuleRange: Module range: %p-%p\n",
                    (void *)moduleBase, (void *)(moduleBase + moduleSize));

  return moduleBase != 0 && moduleSize != 0;
}

// Helper function to format an address
static std::string FormatAddress(duint addr)
{
  char addrText[20] = "";
  if (addr == 0)
    return "00000000";

#ifdef _WIN64
  sprintf_s(addrText, sizeof(addrText), "%016llX", addr);
#else
  sprintf_s(addrText, sizeof(addrText), "%08X", addr);
#endif
  return std::string(addrText);
}

// Helper to get destination address if possible
static bool GetDestAddress(duint address, duint &dest)
{
  dest = 0;
  if (!address)
    return false;

  // Try to use DbgArgumentGet first - most reliable way
  if (DbgArgumentGet(address, 0, &dest) && dest != 0)
    return true;

  // Try to use DbgGetBranchDestination for jumps and calls
  // Функция принимает только адрес и возвращает адрес назначения
  duint branchDest = DbgGetBranchDestination(address);
  if (branchDest != 0)
  {
    dest = branchDest;
    return true;
  }

  // If all else fails, try to parse from disassembly string
  char command[MAX_DISASM_BUFFER] = "";
  if (GuiGetDisassembly(address, command))
  {
    DebugLog("[InstrLogger] Parsing disassembly: %s\n", command);

    // Find address in the command
    char *ptr = command;
    while (*ptr && *ptr != ' ')
      ptr++; // Skip instruction name

    if (*ptr)
    {
      while (*ptr == ' ')
        ptr++; // Skip spaces

      // Try to parse address from various formats

      // Check for hex format with 0x prefix
      if ((strncmp(ptr, "0x", 2) == 0 || strncmp(ptr, "0X", 2) == 0))
      {
        dest = strtoull(ptr + 2, NULL, 16);
        if (dest != 0)
          return true;
      }

      // Check for direct hex format without 0x prefix
      char *endPtr = nullptr;
      dest = strtoull(ptr, &endPtr, 16);
      if (dest != 0 && endPtr != ptr)
        return true;
    }
  }

  return false;
}

InstrLogger::InstrLogger()
    : m_isLogging(false)
{
  // Default options
  m_options.logCalls = true;
  m_options.logJumps = true;
  m_options.logReturns = true;
  m_options.logOnlyCurrentModule = true;
  m_options.followInCpu = false;
  m_options.logToFile = false;

  // Используем правильное расширение файла в зависимости от архитектуры
#ifdef _WIN64
  m_options.outputFile = "instrlog.trace64";
#else
  m_options.outputFile = "instrlog.trace32";
#endif
}

InstrLogger::~InstrLogger()
{
  Shutdown();
}

bool InstrLogger::Initialize()
{
  // Initialize debug log
  InitDebugLog();
  DebugLog("[InstrLogger] Initializing...\n");

  // Get dbg functions
  if (!DbgFunctions())
  {
    DebugLog("[InstrLogger] Failed to get debug functions\n");
    return false;
  }

  // Copy dbg functions
  memcpy(&g_dbgFunctions, DbgFunctions(), sizeof(DBGFUNCTIONS));
  DebugLog("[InstrLogger] DbgFunctions initialized\n");

  // Clear any existing log
  m_logEntries.clear();
  m_moduleCache.clear();

  // Initialize our options to defaults
  m_options.logCalls = true;
  m_options.logJumps = true;
  m_options.logReturns = false;
  m_options.logOnlyCurrentModule = false;
  m_options.followInCpu = false;
  m_options.logToFile = false;

  // Default output file
#ifdef _WIN64
  m_options.outputFile = "instrlog.trace64";
#else
  m_options.outputFile = "instrlog.trace32";
#endif

  // Get the plugin directory for ini file
  std::string pluginDir = GetPluginDirectory();
  if (!pluginDir.empty())
  {
    std::string iniPath = pluginDir + CUSTOM_INI_FILENAME;
    DebugLog("[InstrLogger] INI file path: %s\n", iniPath.c_str());

    // Check if the INI file exists, create it if it doesn't
    DWORD attrs = GetFileAttributesA(iniPath.c_str());
    if (attrs == INVALID_FILE_ATTRIBUTES)
    {
      // File doesn't exist, create it with default settings
      DebugLog("[InstrLogger] INI file doesn't exist, creating it\n");

      // Create an empty file
      HANDLE hFile = CreateFileA(iniPath.c_str(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
                                 FILE_ATTRIBUTE_NORMAL, NULL);
      if (hFile != INVALID_HANDLE_VALUE)
      {
        CloseHandle(hFile);
        DebugLog("[InstrLogger] Created empty INI file\n");

        // Save default settings
        SaveSettings();
      }
      else
      {
        DWORD error = GetLastError();
        DebugLog("[InstrLogger] Failed to create INI file, error: %lu\n", error);
        _plugin_logprintf("[InstrLogger] WARNING: Failed to create settings file: %s (error %lu)\n",
                          iniPath.c_str(), error);
      }
    }
    else
    {
      // File exists, load settings
      DebugLog("[InstrLogger] INI file exists, loading settings\n");
      LoadSettings();
    }
  }
  else
  {
    DebugLog("[InstrLogger] Failed to get plugin directory, using default settings\n");
    _plugin_logprintf("[InstrLogger] WARNING: Failed to get plugin directory, using default settings\n");
  }

  DebugLog("[InstrLogger] Initialized successfully\n");
  return true;
}

void InstrLogger::Shutdown()
{
  // Stop logging if active
  if (m_isLogging)
    StopLogging();

  // Save settings
  SaveSettings();

  // Clear resources
  m_logEntries.clear();
  m_moduleCache.clear();
}

void InstrLogger::OnBreakpoint(PLUG_CB_BREAKPOINT *info)
{
  // Process breakpoint events if logging is active
  if (m_isLogging && info && info->breakpoint)
  {
    duint address = info->breakpoint->addr;
    AddLogEntry(address);
  }
}

void InstrLogger::OnStepped(PLUG_CB_STEPPED *info)
{
  // Process step events if logging is active
  if (m_isLogging)
  {
    duint address = Script::Register::GetCIP();
    AddLogEntry(address);
  }
}

void InstrLogger::OnTraceExecute(PLUG_CB_TRACEEXECUTE *info)
{
  // Process trace execute events if logging is active
  // Don't rely on info->address as it might not exist in some SDK versions
  if (m_isLogging)
  {
    // Always get current instruction pointer directly from the API
    duint address = Script::Register::GetCIP();
    if (address)
      AddLogEntry(address);
  }
}

bool InstrLogger::StartLogging()
{
  DebugLog("[InstrLogger] Starting logging process...\n");

  if (m_isLogging)
  {
    DebugLog("[InstrLogger] Already logging, nothing to do\n");
    return true; // Already logging
  }

  // Open log file if enabled
  if (m_options.logToFile && !m_options.outputFile.empty())
  {
    // Get plugin path and create log file next to the plugin
    char pluginPath[MAX_PATH] = "";
    if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
        GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
    {

      // Extract directory from plugin path
      char *lastSlash = strrchr(pluginPath, '\\');
      if (lastSlash)
      {
        *(lastSlash + 1) = 0; // Truncate to directory path
        std::string logPath = std::string(pluginPath) + m_options.outputFile;
        DebugLog("[InstrLogger] Using plugin directory for log file: %s\n", logPath.c_str());

        try
        {
          // Change from trunc to app mode to append data instead of overwriting
          m_logFile.open(logPath, std::ios::out | std::ios::app);

          // If file is newly created, write the header
          if (m_logFile.is_open() && m_logFile.tellp() == 0)
          {
            m_logFile << "Address,Thread,Module,Instruction,Destination,DestModule,DestSymbol,Time" << std::endl;
            DebugLog("[InstrLogger] Created new log file with header\n");
          }
        }
        catch (const std::exception &e)
        {
          DebugLog("[InstrLogger] Exception when opening log file: %s\n", e.what());
        }

        if (!m_logFile.is_open())
        {
          DebugLog("[InstrLogger] Failed to open log file in plugin dir, falling back to default location\n");
          try
          {
            // Change from trunc to app mode here too
            m_logFile.open(m_options.outputFile, std::ios::out | std::ios::app);

            // If file is newly created, write the header
            if (m_logFile.is_open() && m_logFile.tellp() == 0)
            {
              m_logFile << "Address,Thread,Module,Instruction,Destination,DestModule,DestSymbol,Time" << std::endl;
              DebugLog("[InstrLogger] Created new log file with header (default location)\n");
            }
          }
          catch (const std::exception &e)
          {
            DebugLog("[InstrLogger] Exception when opening fallback log file: %s\n", e.what());
          }
        }
      }
      else
      {
        DebugLog("[InstrLogger] Could not extract plugin directory, using default location\n");
        try
        {
          m_logFile.open(m_options.outputFile, std::ios::out | std::ios::trunc);
        }
        catch (const std::exception &e)
        {
          DebugLog("[InstrLogger] Exception when opening log file: %s\n", e.what());
        }
      }
    }
    else
    {
      DebugLog("[InstrLogger] Could not get plugin path (GetModuleFileName error: %d), using default location\n", GetLastError());
      try
      {
        m_logFile.open(m_options.outputFile, std::ios::out | std::ios::trunc);
      }
      catch (const std::exception &e)
      {
        DebugLog("[InstrLogger] Exception when opening log file: %s\n", e.what());
      }
    }

    if (!m_logFile.is_open())
    {
      DebugLog("[InstrLogger] Failed to open log file: %s\n", m_options.outputFile.c_str());
      return false;
    }
    DebugLog("[InstrLogger] Log file opened successfully\n");

    // Write header
    try
    {
      m_logFile << "Address,Thread,Module,Instruction,Destination,DestModule,DestSymbol,Time" << std::endl;
    }
    catch (const std::exception &e)
    {
      DebugLog("[InstrLogger] Exception when writing log header: %s\n", e.what());
      return false;
    }
  }

  // If module-only logging is enabled, add current module to range
  if (m_options.logOnlyCurrentModule && m_options.moduleRanges.empty())
  {
    DebugLog("[InstrLogger] Module-only logging enabled, determining current module range\n");
    duint moduleBase, moduleSize;
    if (GetCurrentModuleRange(moduleBase, moduleSize))
    {
      DebugLog("[InstrLogger] Found current module range: %p-%p\n",
               (void *)moduleBase, (void *)(moduleBase + moduleSize));
      try
      {
        m_options.moduleRanges.push_back(std::make_pair(moduleBase, moduleBase + moduleSize));
      }
      catch (const std::exception &e)
      {
        DebugLog("[InstrLogger] Exception when adding module range: %s\n", e.what());
      }
    }
    else
    {
      DebugLog("[InstrLogger] Failed to get current module range\n");
      // Even if we fail, we'll continue with logging but will use a fallback approach
      // Just disable module-only logging to allow all instructions to be logged
      DebugLog("[InstrLogger] Falling back to logging all instructions\n");
      m_options.logOnlyCurrentModule = false;
    }
  }

  DebugLog("[InstrLogger] Setting logging state to active\n");
  m_isLogging = true;
  DebugLog("[InstrLogger] Logging started successfully\n");
  return true;
}

bool InstrLogger::StopLogging()
{
  DebugLog("[InstrLogger] Stopping logging process...\n");

  if (!m_isLogging)
  {
    DebugLog("[InstrLogger] Not currently logging, nothing to do\n");
    return true; // Not logging
  }

  m_isLogging = false;

  // Close log file if open
  if (m_logFile.is_open())
  {
    try
    {
      m_logFile.close();
      DebugLog("[InstrLogger] Log file closed\n");
    }
    catch (const std::exception &e)
    {
      DebugLog("[InstrLogger] Exception when closing log file: %s\n", e.what());
    }
  }

  DebugLog("[InstrLogger] Logging stopped\n");
  return true;
}

bool InstrLogger::IsLogging() const
{
  return m_isLogging;
}

void InstrLogger::SetOptions(const LogOptions &options)
{
  m_options = options;
}

const LogOptions &InstrLogger::GetOptions() const
{
  return m_options;
}

void InstrLogger::AddLogEntry(duint address)
{
  // Skip if address not in range or invalid
  if (!address)
    return;

  // Add critical section lock for thread safety
  std::lock_guard<std::mutex> lock(m_instrMutex);

  // Skip if module filtering is on and address is not in current module
  if (m_options.logOnlyCurrentModule && !IsAddressInRange(address))
  {
    // DebugLog("[InstrLogger] Skipping address %p (not in range)\n", (void *)address);
    return;
  }

  // Get instruction information
  std::string instruction;
  char disasm[MAX_DISASM_BUFFER] = "";

  bool gotDisasm = false;
  try
  {
    // Используем только GuiGetDisassembly, так как DbgGetDisassembly не существует
    gotDisasm = GuiGetDisassembly(address, disasm);
  }
  catch (...)
  {
    DebugLog("[InstrLogger] Exception occurred during disassembly at address %p\n", (void *)address);
    return;
  }

  if (!gotDisasm || disasm[0] == '\0')
  {
    DebugLog("[InstrLogger] Could not disassemble instruction at %p\n", (void *)address);
    return; // Could not disassemble
  }

  instruction = disasm;

  // Check if this is an instruction we want to log
  bool isCall = false;
  bool isJump = false;
  bool isReturn = false;

  // Use the improved instruction detection methods
  if (m_options.logCalls)
    isCall = IsCallInstruction(instruction.c_str());

  if (m_options.logJumps)
    isJump = IsJumpInstruction(instruction.c_str());

  if (m_options.logReturns)
    isReturn = IsRetInstruction(instruction.c_str());

  if (!isCall && !isJump && !isReturn)
    return; // Not an instruction we want to log

  // Get destination information for call or jump
  duint destAddress = 0;
  std::string destModule = "";
  std::string destSymbol = "";

  if (isCall || isJump)
  {
    try
    {
      // Use our improved GetDestAddress function to get the destination
      if (GetDestAddress(address, destAddress) && destAddress != 0)
      {
        // Check if the destination address is valid before trying to get additional info
        if (DbgMemIsValidReadPtr(destAddress))
        {
          destModule = GetModuleNameFromAddr(destAddress);
          destSymbol = GetLabelFromAddress(destAddress);
        }
        else
        {
          DebugLog("[InstrLogger] Destination address %p is not valid\n", (void *)destAddress);
        }
      }
      else
      {
        DebugLog("[InstrLogger] Could not get destination for %s at %p\n",
                 instruction.c_str(), (void *)address);
      }
    }
    catch (const std::exception &e)
    {
      DebugLog("[InstrLogger] Exception when getting destination: %s\n", e.what());
      destAddress = 0;
    }
    catch (...)
    {
      DebugLog("[InstrLogger] Unknown exception when getting destination\n");
      destAddress = 0;
    }
  }

  // Create log record
  LogRecord record;
  record.address = address;
  record.instruction = instruction;
  record.destAddress = destAddress;
  record.destModule = destModule;
  record.destSymbol = destSymbol;
  record.threadId = DbgGetThreadId();
  GetSystemTime(&record.timestamp);

  // Add to memory log
  m_logEntries.push_back(record);

  // Write to log file if enabled
  if (m_options.logToFile && m_logFile.is_open())
  {
    try
    {
      char time[20];
      sprintf_s(time, sizeof(time), "%02d:%02d:%02d.%03d",
                record.timestamp.wHour, record.timestamp.wMinute,
                record.timestamp.wSecond, record.timestamp.wMilliseconds);

      m_logFile << FormatAddress(record.address) << ","
                << record.threadId << ","
                << GetModuleNameFromAddr(record.address) << ","
                << "\"" << record.instruction << "\","
                << FormatAddress(record.destAddress) << ","
                << destModule << ","
                << "\"" << record.destSymbol << "\","
                << time << std::endl;

      // Обеспечиваем запись в файл
      m_logFile.flush();
    }
    catch (const std::exception &e)
    {
      DebugLog("[InstrLogger] Exception when writing to log file: %s\n", e.what());
    }
    catch (...)
    {
      DebugLog("[InstrLogger] Unknown exception when writing to log file\n");
    }
  }

  // Print to log window, но не перегружаем его
  static int logCount = 0;
  if (++logCount % 50 == 0)
  {
    _plugin_logprintf("[%s] Logged %d instructions so far\n", PLUGIN_NAME, m_logEntries.size());
  }

  // Если включено следование за CPU, передаем управление в дизассемблер
  if (m_options.followInCpu && (isCall || isJump) && destAddress)
  {
    GuiDisasmAt(destAddress, GUI_DISASM_CENTERED);
  }
}

void InstrLogger::ClearLog()
{
  DebugLog("[InstrLogger] Clearing log\n");

  try
  {
    // Lock the log for thread safety
    std::lock_guard<std::mutex> lock(m_instrMutex);

    // Clear the instruction logs (both m_instrLog and m_logEntries)
    try
    {
      // Clear m_instrLog if it exists in the class
      if (m_instrLog.size() > 0)
      {
        m_instrLog.clear();
        DebugLog("[InstrLogger] m_instrLog cleared in memory\n");
      }

      // Clear m_logEntries which is the main storage
      m_logEntries.clear();
      DebugLog("[InstrLogger] Log entries cleared in memory\n");
    }
    catch (const std::exception &e)
    {
      DebugLog("[InstrLogger] Exception when clearing logs: %s\n", e.what());
    }

    // Clear file if it's open
    if (m_logFile.is_open())
    {
      try
      {
        // Close and reopen to truncate
        m_logFile.close();

        // Try to reopen if logging is active
        if (m_isLogging && m_options.logToFile && !m_options.outputFile.empty())
        {
          try
          {
            // Changed from trunc mode to out to maintain consistency with StartLogging
            m_logFile.open(m_options.outputFile, std::ios::out | std::ios::trunc);

            if (m_logFile.is_open())
            {
              // Write header
              m_logFile << "Address,Thread,Module,Instruction,Destination,DestModule,DestSymbol,Time" << std::endl;
              DebugLog("[InstrLogger] Log file cleared and header written\n");
            }
            else
            {
              DebugLog("[InstrLogger] Failed to reopen log file after clearing\n");
            }
          }
          catch (const std::exception &e)
          {
            DebugLog("[InstrLogger] Exception when reopening log file: %s\n", e.what());
          }
        }
        else
        {
          DebugLog("[InstrLogger] Log file closed\n");
        }
      }
      catch (const std::exception &e)
      {
        DebugLog("[InstrLogger] Exception when closing log file: %s\n", e.what());
      }
    }
  }
  catch (const std::exception &e)
  {
    DebugLog("[InstrLogger] Exception in ClearLog: %s\n", e.what());
  }
  catch (...)
  {
    DebugLog("[InstrLogger] Unknown exception in ClearLog\n");
  }
}

const std::vector<LogRecord> &InstrLogger::GetLogEntries() const
{
  return m_logEntries;
}

bool InstrLogger::SaveLog(const std::string &filePath)
{
  std::string path = filePath.empty() ? m_options.outputFile : filePath;
  DebugLog("[InstrLogger] Saving log to file: %s\n", path.c_str());

  // Get plugin path and save log file next to the plugin
  char pluginPath[MAX_PATH] = "";
  bool usePluginDir = false;
  std::string fullPath = path;

  if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
      GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
  {

    // Extract directory from plugin path
    char *lastSlash = strrchr(pluginPath, '\\');
    if (lastSlash)
    {
      *(lastSlash + 1) = 0; // Truncate to directory path
      fullPath = std::string(pluginPath) + path;
      usePluginDir = true;

    }
  }

  if (!usePluginDir)
  {
    DebugLog("[InstrLogger] Could not use plugin directory, saving to: %s\n", path.c_str());
  }

  // Check if we're saving to our active logging file
  bool isActiveLogFile = m_isLogging && m_options.logToFile &&
                        (fullPath == m_options.outputFile || path == m_options.outputFile);

  if (isActiveLogFile && m_logFile.is_open())
  {
    // If we're already logging to this file, just flush any buffered data
    try
    {
      m_logFile.flush();
      DebugLog("[InstrLogger] Flushed current log file\n");
      _plugin_logprintf("[%s] Log file %s updated (real-time logging active)\n", PLUGIN_NAME, fullPath.c_str());
      return true;
    }
    catch (const std::exception &e)
    {
      DebugLog("[InstrLogger] Exception when flushing log file: %s\n", e.what());
      // Continue to normal save process as fallback
    }
  }

  std::ofstream file;
  try
  {
    // If this is a new file (not our active log), use trunc mode as before
    file.open(fullPath, std::ios::out | std::ios::trunc);
  }
  catch (const std::exception &e)
  {
    DebugLog("[InstrLogger] Exception when opening log file: %s\n", e.what());
  }

  if (!file.is_open())
  {
    DebugLog("[InstrLogger] Failed to open log file: %s\n", fullPath.c_str());

    // Try default path if plugin directory didn't work
    if (usePluginDir)
    {
      DebugLog("[InstrLogger] Trying default path instead: %s\n", path.c_str());
      try
      {
        file.open(path, std::ios::out | std::ios::trunc);
      }
      catch (const std::exception &e)
      {
        DebugLog("[InstrLogger] Exception when opening default path log file: %s\n", e.what());
      }

      if (!file.is_open())
      {
        DebugLog("[InstrLogger] Also failed to open log file at default path\n");
        return false;
      }

      // If we got here, use the original path for log messages
      fullPath = path;
    }
    else
    {
      return false;
    }
  }

  // Write header
  try
  {
    file << "Address,Thread,Module,Instruction,Destination,DestModule,DestSymbol,Time" << std::endl;
  }
  catch (const std::exception &e)
  {
    DebugLog("[InstrLogger] Exception when writing log header: %s\n", e.what());
    file.close();
    return false;
  }

  // Write each instruction
  std::lock_guard<std::mutex> lock(m_instrMutex);
  try
  {
    // First try to write from m_instrLog if it's not empty
    if (!m_instrLog.empty())
    {
      DebugLog("[InstrLogger] Writing from m_instrLog (%zu entries)\n", m_instrLog.size());
      for (const auto &instr : m_instrLog)
      {
        if (instr.type == InstrType::NORMAL || instr.type == InstrType::CALL)
        {
          // Fix for hexadecimal output - use string formatting instead of << operator
          file << std::hex << "0x" << instr.address << std::dec
               << "," << instr.threadId
               << "," << instr.module
               << "," << instr.instruction;

          // Destination address - only output if non-zero
          if (instr.destination)
          {
            file << "," << std::hex << "0x" << instr.destination << std::dec;
          }

          file << "," << instr.destModule
               << "," << instr.destSymbol
               << "," << instr.timeStamp.count()
               << std::endl;
        }
      }
    }
    // Also write from m_logEntries if it's not empty
    else if (!m_logEntries.empty())
    {
      DebugLog("[InstrLogger] Writing from m_logEntries (%zu entries)\n", m_logEntries.size());
      for (const auto &record : m_logEntries)
      {
        // Format timestamp
        char time[20];
        sprintf_s(time, sizeof(time), "%02d:%02d:%02d.%03d",
                  record.timestamp.wHour, record.timestamp.wMinute,
                  record.timestamp.wSecond, record.timestamp.wMilliseconds);

        file << FormatAddress(record.address) << ","
             << record.threadId << ","
             << GetModuleNameFromAddr(record.address) << ","
             << "\"" << record.instruction << "\","
             << FormatAddress(record.destAddress) << ","
             << record.destModule << ","
             << "\"" << record.destSymbol << "\","
             << time << std::endl;
      }
    }
    else
    {
      DebugLog("[InstrLogger] No log entries to write\n");
    }
  }
  catch (const std::exception &e)
  {
    DebugLog("[InstrLogger] Exception when writing log entries: %s\n", e.what());
    file.close();
    return false;
  }

  file.close();
  DebugLog("[InstrLogger] Log saved to %s\n", fullPath.c_str());
  return true;
}

bool InstrLogger::SaveSettings()
{
  DebugLog("[%s] Saving settings to custom INI file\n", PLUGIN_NAME);

  // Get INI file path for logging
  std::string pluginDir = GetPluginDirectory();
  if (pluginDir.empty())
  {
    DebugLog("[%s] ERROR: Cannot save settings - plugin directory not found\n", PLUGIN_NAME);
    _plugin_logprintf("[%s] ERROR: Cannot save settings - plugin directory not found\n", PLUGIN_NAME);
    return false;
  }

  std::string iniPath = pluginDir + CUSTOM_INI_FILENAME;
  DebugLog("[%s] Using INI file: %s\n", PLUGIN_NAME, iniPath.c_str());

  // Ensure the file is writable before attempting to save
  HANDLE hFile = CreateFileA(iniPath.c_str(), GENERIC_READ | GENERIC_WRITE, 0, NULL,
                             OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
  if (hFile == INVALID_HANDLE_VALUE)
  {
    DWORD error = GetLastError();
    DebugLog("[%s] ERROR: Cannot open INI file for writing, error: %lu\n", PLUGIN_NAME, error);
    _plugin_logprintf("[%s] ERROR: Cannot open settings file for writing: %s\n", PLUGIN_NAME, iniPath.c_str());
    return false;
  }
  CloseHandle(hFile);

  // First make a backup of the INI file if it exists
  DWORD fileAttrs = GetFileAttributesA(iniPath.c_str());
  if (fileAttrs != INVALID_FILE_ATTRIBUTES)
  {
    std::string backupPath = iniPath + ".bak";
    DebugLog("[%s] Creating backup of INI file: %s\n", PLUGIN_NAME, backupPath.c_str());

    // Delete old backup if it exists
    DeleteFileA(backupPath.c_str());

    // Copy current INI to backup
    if (!CopyFileA(iniPath.c_str(), backupPath.c_str(), FALSE))
    {
      DWORD error = GetLastError();
      DebugLog("[%s] Warning: Failed to create backup, error: %lu\n", PLUGIN_NAME, error);
      // Continue anyway
    }
  }

  bool success = true;
  char buffer[32];

  // Convert bool values to strings
  sprintf_s(buffer, "%d", m_options.logCalls ? 1 : 0);
  bool result = SaveSettingToIni(SETTINGS_SECTION, "LogCalls", buffer);
  DebugLog("[%s] Save LogCalls=%s: %s\n", PLUGIN_NAME, buffer, result ? "Success" : "Failed");
  success &= result;

  sprintf_s(buffer, "%d", m_options.logJumps ? 1 : 0);
  result = SaveSettingToIni(SETTINGS_SECTION, "LogJumps", buffer);
  DebugLog("[%s] Save LogJumps=%s: %s\n", PLUGIN_NAME, buffer, result ? "Success" : "Failed");
  success &= result;

  sprintf_s(buffer, "%d", m_options.logReturns ? 1 : 0);
  result = SaveSettingToIni(SETTINGS_SECTION, "LogReturns", buffer);
  DebugLog("[%s] Save LogReturns=%s: %s\n", PLUGIN_NAME, buffer, result ? "Success" : "Failed");
  success &= result;

  sprintf_s(buffer, "%d", m_options.logOnlyCurrentModule ? 1 : 0);
  result = SaveSettingToIni(SETTINGS_SECTION, "LogOnlyCurrentModule", buffer);
  DebugLog("[%s] Save LogOnlyCurrentModule=%s: %s\n", PLUGIN_NAME, buffer, result ? "Success" : "Failed");
  success &= result;

  sprintf_s(buffer, "%d", m_options.followInCpu ? 1 : 0);
  result = SaveSettingToIni(SETTINGS_SECTION, "FollowInCpu", buffer);
  DebugLog("[%s] Save FollowInCpu=%s: %s\n", PLUGIN_NAME, buffer, result ? "Success" : "Failed");
  success &= result;

  sprintf_s(buffer, "%d", m_options.logToFile ? 1 : 0);
  result = SaveSettingToIni(SETTINGS_SECTION, "LogToFile", buffer);
  DebugLog("[%s] Save LogToFile=%s: %s\n", PLUGIN_NAME, buffer, result ? "Success" : "Failed");
  success &= result;

  // Save output file path
  result = SaveSettingToIni(SETTINGS_SECTION, "OutputFile", m_options.outputFile.c_str());
  DebugLog("[%s] Save OutputFile=%s: %s\n", PLUGIN_NAME, m_options.outputFile.c_str(), result ? "Success" : "Failed");
  success &= result;

  // Сохраняем количество шагов трассировки
  sprintf_s(buffer, "%d", m_options.traceStepCount);
  result = SaveSettingToIni(SETTINGS_SECTION, "TraceStepCount", buffer);
  DebugLog("[%s] Save TraceStepCount=%s (%d): %s\n", PLUGIN_NAME, buffer, m_options.traceStepCount, result ? "Success" : "Failed");
  success &= result;

  // Additional meta-information for the plugin
  sprintf_s(buffer, "%d", 1); // Settings version = 1
  result = SaveSettingToIni(SETTINGS_SECTION, "SettingsVersion", buffer);
  DebugLog("[%s] Save SettingsVersion=%s: %s\n", PLUGIN_NAME, buffer, result ? "Success" : "Failed");
  success &= result;

  // Timestamp when settings were saved
  SYSTEMTIME st;
  GetLocalTime(&st);
  char timestamp[64];
  sprintf_s(timestamp, sizeof(timestamp), "%04d-%02d-%02d %02d:%02d:%02d",
            st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
  result = SaveSettingToIni(SETTINGS_SECTION, "LastSaved", timestamp);
  DebugLog("[%s] Save LastSaved=%s: %s\n", PLUGIN_NAME, timestamp, result ? "Success" : "Failed");
  success &= result;

  // Force the system to flush the cache to make sure settings are written
  WritePrivateProfileStringA(NULL, NULL, NULL, iniPath.c_str());

  DebugLog("[%s] Settings %s to %s\n", PLUGIN_NAME,
           success ? "saved successfully" : "save failed",
           iniPath.c_str());

  // Verify settings were saved correctly by doing an immediate reload
  if (success)
  {
    DebugLog("[%s] Verifying saved settings by reloading from INI\n", PLUGIN_NAME);

    // Store current settings
    LogOptions originalOptions = m_options;

    // Clear settings and reload
    m_options = LogOptions(); // Reset to defaults
    bool verifyResult = LoadSettings();

    if (verifyResult)
    {
      // Compare all settings
      bool allMatch =
          (originalOptions.logCalls == m_options.logCalls) &&
          (originalOptions.logJumps == m_options.logJumps) &&
          (originalOptions.logReturns == m_options.logReturns) &&
          (originalOptions.logOnlyCurrentModule == m_options.logOnlyCurrentModule) &&
          (originalOptions.followInCpu == m_options.followInCpu) &&
          (originalOptions.logToFile == m_options.logToFile) &&
          (originalOptions.outputFile == m_options.outputFile);

      if (!allMatch)
      {
        DebugLog("[%s] ERROR: Settings verification failed! Values don't match after reload.\n", PLUGIN_NAME);

        // Log differences
        if (originalOptions.logCalls != m_options.logCalls)
          DebugLog("[%s]   LogCalls: %d != %d\n", PLUGIN_NAME, originalOptions.logCalls, m_options.logCalls);
        if (originalOptions.logJumps != m_options.logJumps)
          DebugLog("[%s]   LogJumps: %d != %d\n", PLUGIN_NAME, originalOptions.logJumps, m_options.logJumps);
        if (originalOptions.logReturns != m_options.logReturns)
          DebugLog("[%s]   LogReturns: %d != %d\n", PLUGIN_NAME, originalOptions.logReturns, m_options.logReturns);
        if (originalOptions.logOnlyCurrentModule != m_options.logOnlyCurrentModule)
          DebugLog("[%s]   LogOnlyCurrentModule: %d != %d\n", PLUGIN_NAME, originalOptions.logOnlyCurrentModule, m_options.logOnlyCurrentModule);
        if (originalOptions.followInCpu != m_options.followInCpu)
          DebugLog("[%s]   FollowInCpu: %d != %d\n", PLUGIN_NAME, originalOptions.followInCpu, m_options.followInCpu);
        if (originalOptions.logToFile != m_options.logToFile)
          DebugLog("[%s]   LogToFile: %d != %d\n", PLUGIN_NAME, originalOptions.logToFile, m_options.logToFile);
        if (originalOptions.outputFile != m_options.outputFile)
          DebugLog("[%s]   OutputFile: %s != %s\n", PLUGIN_NAME, originalOptions.outputFile.c_str(), m_options.outputFile.c_str());

        // Restore original settings and return false
        m_options = originalOptions;
        success = false;
      }
      else
      {
        DebugLog("[%s] Settings verification successful - all values match after reload\n", PLUGIN_NAME);
      }
    }
    else
    {
      DebugLog("[%s] ERROR: Could not reload settings for verification!\n", PLUGIN_NAME);

      // Restore original settings and return false
      m_options = originalOptions;
      success = false;
    }
  }

  // Also log in x64dbg
  _plugin_logprintf("[%s] Settings %s to %s\n", PLUGIN_NAME,
                    success ? "saved successfully" : "save failed",
                    iniPath.c_str());

  return success;
}

bool InstrLogger::LoadSettings()
{
  DebugLog("[%s] Loading settings from custom INI file\n", PLUGIN_NAME);

  // Ensure we're using the same path as in SaveSettings
  std::string pluginDir = GetPluginDirectory();
  if (pluginDir.empty())
  {
    DebugLog("[%s] ERROR: Failed to get plugin directory for INI file\n", PLUGIN_NAME);
    return false;
  }

  std::string iniPath = pluginDir + CUSTOM_INI_FILENAME;
  DebugLog("[%s] Loading settings from INI file: %s\n", PLUGIN_NAME, iniPath.c_str());

  // Check if file exists
  DWORD attrs = GetFileAttributesA(iniPath.c_str());
  if (attrs == INVALID_FILE_ATTRIBUTES)
  {
    DebugLog("[%s] INI file not found at: %s\n", PLUGIN_NAME, iniPath.c_str());
    _plugin_logprintf("[%s] Settings file not found at: %s\n", PLUGIN_NAME, iniPath.c_str());

    // Set default values and save immediately for next time
    m_options.logCalls = true;
    m_options.logJumps = true;
    m_options.logReturns = false;
    m_options.logOnlyCurrentModule = false;
    m_options.followInCpu = false;
    m_options.logToFile = false;

    // Default output file
#ifdef _WIN64
    m_options.outputFile = "instrlog.trace64";
#else
    m_options.outputFile = "instrlog.trace32";
#endif

    // Save defaults immediately for next time
    DebugLog("[%s] Saving default settings\n", PLUGIN_NAME);
    SaveSettings();
    return true;
  }

  // Try to open the file directly to verify it's readable
  HANDLE hFile = CreateFileA(iniPath.c_str(), GENERIC_READ, FILE_SHARE_READ, NULL,
                             OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if (hFile == INVALID_HANDLE_VALUE)
  {
    DWORD error = GetLastError();
    DebugLog("[%s] ERROR: INI file cannot be opened for reading: %s (error: %lu)\n",
             PLUGIN_NAME, iniPath.c_str(), error);

    // Use default values
    m_options.logCalls = true;
    m_options.logJumps = true;
    m_options.logReturns = false;
    m_options.logOnlyCurrentModule = false;
    m_options.followInCpu = false;
    m_options.logToFile = false;

#ifdef _WIN64
    m_options.outputFile = "instrlog.trace64";
#else
    m_options.outputFile = "instrlog.trace32";
#endif

    return false;
  }
  CloseHandle(hFile);

  bool settingsFound = false;
  char buffer[MAX_PATH] = "";

  // Check if settings file exists by reading version
  if (LoadSettingFromIni(SETTINGS_SECTION, "SettingsVersion", buffer, sizeof(buffer)))
  {
    int version = atoi(buffer);
    settingsFound = version > 0;
    DebugLog("[%s] Found saved settings (version: %d) in %s\n", PLUGIN_NAME, version, iniPath.c_str());
  }

  // Store original settings for verification
  LogOptions originalOptions = m_options;
  bool settingsChanged = false;

  // Load each setting with default values if not found
  DebugLog("[%s] Loading LogCalls setting\n", PLUGIN_NAME);
  if (LoadSettingFromIni(SETTINGS_SECTION, "LogCalls", buffer, sizeof(buffer), "1"))
  {
    bool newValue = atoi(buffer) != 0;
    settingsChanged = settingsChanged || (m_options.logCalls != newValue);
    m_options.logCalls = newValue;
    DebugLog("[%s] Loaded LogCalls = %s\n", PLUGIN_NAME, m_options.logCalls ? "Yes" : "No");
    settingsFound = true;
  }

  DebugLog("[%s] Loading LogJumps setting\n", PLUGIN_NAME);
  if (LoadSettingFromIni(SETTINGS_SECTION, "LogJumps", buffer, sizeof(buffer), "1"))
  {
    bool newValue = atoi(buffer) != 0;
    settingsChanged = settingsChanged || (m_options.logJumps != newValue);
    m_options.logJumps = newValue;
    DebugLog("[%s] Loaded LogJumps = %s\n", PLUGIN_NAME, m_options.logJumps ? "Yes" : "No");
    settingsFound = true;
  }

  DebugLog("[%s] Loading LogReturns setting\n", PLUGIN_NAME);
  if (LoadSettingFromIni(SETTINGS_SECTION, "LogReturns", buffer, sizeof(buffer), "0"))
  {
    bool newValue = atoi(buffer) != 0;
    settingsChanged = settingsChanged || (m_options.logReturns != newValue);
    m_options.logReturns = newValue;
    DebugLog("[%s] Loaded LogReturns = %s\n", PLUGIN_NAME, m_options.logReturns ? "Yes" : "No");
    settingsFound = true;
  }

  DebugLog("[%s] Loading LogOnlyCurrentModule setting\n", PLUGIN_NAME);
  if (LoadSettingFromIni(SETTINGS_SECTION, "LogOnlyCurrentModule", buffer, sizeof(buffer), "0"))
  {
    bool newValue = atoi(buffer) != 0;
    settingsChanged = settingsChanged || (m_options.logOnlyCurrentModule != newValue);
    m_options.logOnlyCurrentModule = newValue;
    DebugLog("[%s] Loaded LogOnlyCurrentModule = %s\n", PLUGIN_NAME, m_options.logOnlyCurrentModule ? "Yes" : "No");
    settingsFound = true;
  }

  DebugLog("[%s] Loading FollowInCpu setting\n", PLUGIN_NAME);
  if (LoadSettingFromIni(SETTINGS_SECTION, "FollowInCpu", buffer, sizeof(buffer), "0"))
  {
    bool newValue = atoi(buffer) != 0;
    settingsChanged = settingsChanged || (m_options.followInCpu != newValue);
    m_options.followInCpu = newValue;
    DebugLog("[%s] Loaded FollowInCpu = %s\n", PLUGIN_NAME, m_options.followInCpu ? "Yes" : "No");
    settingsFound = true;
  }

  DebugLog("[%s] Loading LogToFile setting\n", PLUGIN_NAME);
  if (LoadSettingFromIni(SETTINGS_SECTION, "LogToFile", buffer, sizeof(buffer), "0"))
  {
    bool newValue = atoi(buffer) != 0;
    settingsChanged = settingsChanged || (m_options.logToFile != newValue);
    m_options.logToFile = newValue;
    DebugLog("[%s] Loaded LogToFile = %s\n", PLUGIN_NAME, m_options.logToFile ? "Yes" : "No");
    settingsFound = true;
  }

  // Default output file
#ifdef _WIN64
  const char *defaultOutputFile = "instrlog.trace64";
#else
  const char *defaultOutputFile = "instrlog.trace32";
#endif

  DebugLog("[%s] Loading OutputFile setting\n", PLUGIN_NAME);
  if (LoadSettingFromIni(SETTINGS_SECTION, "OutputFile", buffer, sizeof(buffer), defaultOutputFile))
  {
    settingsChanged = settingsChanged || (m_options.outputFile != buffer);
    m_options.outputFile = buffer;
    DebugLog("[%s] Loaded OutputFile = %s\n", PLUGIN_NAME, m_options.outputFile.c_str());
    settingsFound = true;
  }

  // Загружаем количество шагов трассировки
  DebugLog("[%s] Loading TraceStepCount setting\n", PLUGIN_NAME);
  if (LoadSettingFromIni(SETTINGS_SECTION, "TraceStepCount", buffer, sizeof(buffer), "1000000"))
  {
    DebugLog("[%s] Raw TraceStepCount value from INI: '%s'\n", PLUGIN_NAME, buffer);

    // Преобразуем строку в число, проверяя возможные ошибки
    int traceStepCount = 0;
    try {
      traceStepCount = std::stoi(buffer);
      DebugLog("[%s] Converted TraceStepCount from string to int: %d\n", PLUGIN_NAME, traceStepCount);
    }
    catch (const std::exception& e) {
      DebugLog("[%s] Error converting TraceStepCount to integer: %s\n", PLUGIN_NAME, e.what());
      traceStepCount = 1000000; // Значение по умолчанию при ошибке
    }

    // Проверяем диапазон
    if (traceStepCount < 1) {
      DebugLog("[%s] TraceStepCount value too small (%d), setting to default 1000000\n", PLUGIN_NAME, traceStepCount);
      traceStepCount = 1000000; // Если значение некорректное, используем стандартное значение из LogOptions
    }
    if (traceStepCount > 100000000) {
      DebugLog("[%s] TraceStepCount value too large (%d), capping at 100000000\n", PLUGIN_NAME, traceStepCount);
      traceStepCount = 100000000; // Проверка на максимальное значение
    }

    settingsChanged = settingsChanged || (m_options.traceStepCount != traceStepCount);
    m_options.traceStepCount = traceStepCount;
    DebugLog("[%s] Loaded TraceStepCount = %d\n", PLUGIN_NAME, m_options.traceStepCount);
    settingsFound = true;
  }

  if (settingsFound)
  {
    // Get last saved timestamp if available
    if (LoadSettingFromIni(SETTINGS_SECTION, "LastSaved", buffer, sizeof(buffer)))
    {
      DebugLog("[%s] Settings last saved on: %s\n", PLUGIN_NAME, buffer);
    }

    DebugLog("[%s] Settings loaded successfully from %s\n", PLUGIN_NAME, iniPath.c_str());

    // Debug output current settings
    DebugLog("[%s] Current settings after loading:\n", PLUGIN_NAME);
    DebugLog("[%s]   Log CALL instructions: %s\n", PLUGIN_NAME, m_options.logCalls ? "Yes" : "No");
    DebugLog("[%s]   Log JMP instructions: %s\n", PLUGIN_NAME, m_options.logJumps ? "Yes" : "No");
    DebugLog("[%s]   Log RET instructions: %s\n", PLUGIN_NAME, m_options.logReturns ? "Yes" : "No");
    DebugLog("[%s]   Log only current module: %s\n", PLUGIN_NAME, m_options.logOnlyCurrentModule ? "Yes" : "No");
    DebugLog("[%s]   Follow in CPU: %s\n", PLUGIN_NAME, m_options.followInCpu ? "Yes" : "No");
    DebugLog("[%s]   Log to file: %s\n", PLUGIN_NAME, m_options.logToFile ? "Yes" : "No");
    DebugLog("[%s]   Output file: %s\n", PLUGIN_NAME, m_options.outputFile.c_str());
    DebugLog("[%s]   Trace step count: %d\n", PLUGIN_NAME, m_options.traceStepCount);

    // Log to x64dbg output
    _plugin_logprintf("[%s] Settings loaded from %s\n", PLUGIN_NAME, iniPath.c_str());

    // If we detected changes, report this for diagnostic purposes
    if (settingsChanged)
    {
      DebugLog("[%s] Notice: Settings were changed during load\n", PLUGIN_NAME);

      // Show only the values that changed
      if (originalOptions.logCalls != m_options.logCalls)
        DebugLog("[%s]   Changed LogCalls: %s -> %s\n", PLUGIN_NAME,
                 originalOptions.logCalls ? "Yes" : "No", m_options.logCalls ? "Yes" : "No");
      if (originalOptions.logJumps != m_options.logJumps)
        DebugLog("[%s]   Changed LogJumps: %s -> %s\n", PLUGIN_NAME,
                 originalOptions.logJumps ? "Yes" : "No", m_options.logJumps ? "Yes" : "No");
      if (originalOptions.logReturns != m_options.logReturns)
        DebugLog("[%s]   Changed LogReturns: %s -> %s\n", PLUGIN_NAME,
                 originalOptions.logReturns ? "Yes" : "No", m_options.logReturns ? "Yes" : "No");
      if (originalOptions.logOnlyCurrentModule != m_options.logOnlyCurrentModule)
        DebugLog("[%s]   Changed LogOnlyCurrentModule: %s -> %s\n", PLUGIN_NAME,
                 originalOptions.logOnlyCurrentModule ? "Yes" : "No", m_options.logOnlyCurrentModule ? "Yes" : "No");
      if (originalOptions.followInCpu != m_options.followInCpu)
        DebugLog("[%s]   Changed FollowInCpu: %s -> %s\n", PLUGIN_NAME,
                 originalOptions.followInCpu ? "Yes" : "No", m_options.followInCpu ? "Yes" : "No");
      if (originalOptions.logToFile != m_options.logToFile)
        DebugLog("[%s]   Changed LogToFile: %s -> %s\n", PLUGIN_NAME,
                 originalOptions.logToFile ? "Yes" : "No", m_options.logToFile ? "Yes" : "No");
      if (originalOptions.outputFile != m_options.outputFile)
        DebugLog("[%s]   Changed OutputFile: %s -> %s\n", PLUGIN_NAME,
                 originalOptions.outputFile.c_str(), m_options.outputFile.c_str());
    }
  }
  else
  {
    DebugLog("[%s] No valid settings found, using defaults\n", PLUGIN_NAME);
    _plugin_logprintf("[%s] No valid settings found in %s, using defaults\n", PLUGIN_NAME, iniPath.c_str());

    // Set default values
    m_options.logCalls = true;
    m_options.logJumps = true;
    m_options.logReturns = false;
    m_options.logOnlyCurrentModule = false;
    m_options.followInCpu = false;
    m_options.logToFile = false;

    // Default output file
#ifdef _WIN64
    m_options.outputFile = "instrlog.trace64";
#else
    m_options.outputFile = "instrlog.trace32";
#endif

    // Save defaults immediately for next time
    SaveSettings();
  }

  return true;
}

bool InstrLogger::IsAddressInRange(duint address) const
{
  if (!address || !DbgMemIsValidReadPtr(address))
  {
    DebugLog("[InstrLogger] Address %p is invalid or not readable\n", (void *)address);
    return false;
  }

  // Immediately return true if logging all modules
  if (!m_options.logOnlyCurrentModule)
    return true;

  // Check if address is in a module - cache the result for better performance
  auto it = m_moduleCache.find(address);
  if (it != m_moduleCache.end())
    return it->second;

  // Check custom ranges first
  for (const auto &range : m_options.customRanges)
  {
    if (address >= range.first && address < range.second)
    {
      // Cache the result and return
      m_moduleCache[address] = true;
      return true;
    }
  }

  // Check module ranges
  for (const auto &range : m_options.moduleRanges)
  {
    if (address >= range.first && address < range.second)
    {
      // Cache the result and return
      m_moduleCache[address] = true;
      return true;
    }
  }

  // If no ranges are defined, try getting the module info on-the-fly
  if (m_options.moduleRanges.empty() && m_options.customRanges.empty())
  {
    // Check if address is in a module
    char moduleName[MAX_MODULE_SIZE] = "";
    bool inModule = false;

    try
    {
      inModule = DbgGetModuleAt(address, moduleName);
      if (inModule)
      {
        // Cache result and return
        m_moduleCache[address] = true;
        return true;
      }
    }
    catch (...)
    {
      DebugLog("[InstrLogger] Exception when checking if address %p is in module\n", (void *)address);
    }
  }

  // Cache negative result and return
  m_moduleCache[address] = false;
  return false;
}

std::string InstrLogger::FormatInstruction(duint address, const std::string &instruction,
                                           duint destAddress, const std::string &destModule,
                                           const std::string &destSymbol)
{
  std::string result = FormatAddress(address) + ": " + instruction;

  if (destAddress)
  {
    result += " -> " + FormatAddress(destAddress);
    if (!destModule.empty())
      result += " (" + destModule + ")";
    if (!destSymbol.empty())
      result += " " + destSymbol;
  }

  return result;
}

std::string InstrLogger::GetLabelFromAddress(duint addr)
{
  // Валидация адреса
  if (!addr || !DbgMemIsValidReadPtr(addr))
    return "";

  // Проверяем кэш
  {
    std::lock_guard<std::mutex> lock(g_symbolCache_mutex);
    auto it = g_symbolCache.find(addr);
    if (it != g_symbolCache.end())
      return it->second;
  }

  std::string result = "";

  // Попытка получить метку
  char label[MAX_LABEL_SIZE] = "";
  bool hasLabel = false;

  try
  {
    hasLabel = DbgGetLabelAt(addr, SEG_DEFAULT, label);
    if (hasLabel && label[0] != '\0')
    {
      result = std::string(label);

      // Сохраняем в кэш
      std::lock_guard<std::mutex> lock(g_symbolCache_mutex);
      g_symbolCache[addr] = result;
      return result;
    }
  }
  catch (...)
  {
    DebugLog("[InstrLogger] Exception in DbgGetLabelAt for address %p\n", (void *)addr);
  }

  // Проверка наличия символа
  try
  {
    // Проверка, находится ли адрес в модуле
    char moduleName[MAX_MODULE_SIZE] = "";
    bool inModule = DbgGetModuleAt(addr, moduleName);

    if (inModule)
    {
      // Получение символьной информации
      SYMBOLINFO info;
      memset(&info, 0, sizeof(SYMBOLINFO));

      if (DbgGetSymbolInfoAt(addr, &info))
      {
        if (info.decoratedSymbol)
        {
          result = info.decoratedSymbol;
          if (info.freeDecorated)
            BridgeFree(info.decoratedSymbol);
        }
        else if (info.undecoratedSymbol)
        {
          result = info.undecoratedSymbol;
          if (info.freeUndecorated)
            BridgeFree(info.undecoratedSymbol);
        }
      }
    }
  }
  catch (const std::exception &e)
  {
    DebugLog("[InstrLogger] Exception when getting symbol: %s\n", e.what());
  }
  catch (...)
  {
    DebugLog("[InstrLogger] Unknown exception when getting symbol for address %p\n", (void *)addr);
  }

  // Сохраняем в кэш даже пустой результат, чтобы не делать повторные вызовы
  std::lock_guard<std::mutex> lock(g_symbolCache_mutex);
  g_symbolCache[addr] = result;
  return result;
}

// Callback: Step event
static void CB_STEP()
{
  if (!g_instrLogger || !g_instrLogger->IsLogging())
    return;

  try
  {
    // Get the current instruction pointer and thread ID directly from the API
    duint cip = Script::Register::GetCIP();
    duint tid = DbgGetThreadId();

    if (cip != 0)
    {
      g_instrLogger->LogInstruction(cip, tid);
    }
  }
  catch (const std::exception &e)
  {
    DebugLog("[InstrLogger] Exception in CB_STEP: %s\n", e.what());
  }
  catch (...)
  {
    DebugLog("[InstrLogger] Unknown exception in CB_STEP\n");
  }
}

bool InstrLogger::LogInstruction(duint address, duint threadId)
{
  try
  {
    DebugLog("[InstrLogger] LogInstruction called for address 0x%p, thread %d\n", (void *)address, threadId);

    if (!m_isLogging)
    {
      DebugLog("[InstrLogger] Not logging, instruction skipped\n");
      return false;
    }

    // Check if address is in range
    if (!IsAddressInRange(address))
    {
      // Out of range, skip
      DebugLog("[InstrLogger] Address 0x%p not in logging range, skipped\n", (void *)address);
      return false;
    }

    // Get instruction information
    InstrInfo info;
    if (!GetInstructionInfo(address, info))
    {
      DebugLog("[InstrLogger] Failed to get instruction info for address 0x%p\n", (void *)address);
      return false;
    }

    // Set thread ID
    info.threadId = threadId;

    // Set timestamp
    info.timeStamp = std::chrono::duration_cast<std::chrono::milliseconds>(
        std::chrono::system_clock::now().time_since_epoch());

    // Check instruction type filter
    if ((info.type == InstrType::CALL && !m_options.logCalls) ||
        (info.type == InstrType::JMP && !m_options.logJumps) ||
        (info.type == InstrType::RET && !m_options.logReturns))
    {
      DebugLog("[InstrLogger] Instruction type filtered out: %s\n", info.instruction.c_str());
      return false; // Skip this type
    }

    // Add to log
    {
      std::lock_guard<std::mutex> lock(m_instrMutex);

      try
      {
        m_instrLog.push_back(info);
      }
      catch (const std::exception &e)
      {
        DebugLog("[InstrLogger] Exception when adding to log: %s\n", e.what());
        return false;
      }
    }

    // Write to log file if enabled
    if (m_options.logToFile && m_logFile.is_open())
    {
      try
      {
        if (info.type == InstrType::NORMAL || info.type == InstrType::CALL || info.type == InstrType::JMP)
        {
          m_logFile << std::hex << "0x" << info.address << std::dec
                    << "," << info.threadId
                    << "," << info.module
                    << "," << info.instruction
                    << ",";

          // Fix the conditional expression with stream operators
          if (info.destination)
          {
            m_logFile << std::hex << "0x" << info.destination << std::dec;
          }

          m_logFile << "," << info.destModule
                    << "," << info.destSymbol
                    << "," << info.timeStamp.count()
                    << std::endl;

          // Force immediate flush to ensure data is written to the file
          m_logFile.flush();
        }
      }
      catch (const std::exception &e)
      {
        DebugLog("[InstrLogger] Exception when writing to log file: %s\n", e.what());
        return false;
      }
    }

    // Update CPU view if enabled
    if (m_options.followInCpu)
    {
      try
      {
        GuiDisasmAt(address, GUI_DISASM_CENTERED);
      }
      catch (const std::exception &e)
      {
        DebugLog("[InstrLogger] Exception when updating CPU view: %s\n", e.what());
      }
    }

    return true;
  }
  catch (const std::exception &e)
  {
    DebugLog("[InstrLogger] Exception in LogInstruction: %s\n", e.what());
    return false;
  }
  catch (...)
  {
    DebugLog("[InstrLogger] Unknown exception in LogInstruction\n");
    return false;
  }
}

bool InstrLogger::GetInstructionInfo(duint address, InstrInfo &info)
{
  try
  {
    DebugLog("[InstrLogger] GetInstructionInfo for address 0x%p\n", (void *)address);

    // Clear the info struct
    info = InstrInfo();
    info.address = address;

    // Get the module name
    info.module = GetModuleNameFromAddr(address);

    // Get disassembly
    char disasm[MAX_DISASM_BUFFER] = "";
    bool gotDisasm = false;

    try
    {
      gotDisasm = GuiGetDisassembly(address, disasm);
    }
    catch (const std::exception &e)
    {
      DebugLog("[InstrLogger] Exception in GuiGetDisassembly: %s\n", e.what());
      return false;
    }
    catch (...)
    {
      DebugLog("[InstrLogger] Unknown exception in GuiGetDisassembly\n");
      return false;
    }

    if (!gotDisasm)
    {
      DebugLog("[InstrLogger] Failed to get disassembly for address 0x%p\n", (void *)address);
      return false;
    }

    info.instruction = disasm;
    DebugLog("[InstrLogger] Got disassembly: %s\n", info.instruction.c_str());

    // Determine instruction type
    if (IsCallInstruction(info.instruction.c_str()))
    {
      info.type = InstrType::CALL;
      DebugLog("[InstrLogger] Detected CALL instruction\n");
    }
    else if (IsJumpInstruction(info.instruction.c_str()))
    {
      info.type = InstrType::JMP;
      DebugLog("[InstrLogger] Detected JUMP instruction\n");
    }
    else if (IsRetInstruction(info.instruction.c_str()))
    {
      info.type = InstrType::RET;
      DebugLog("[InstrLogger] Detected RETURN instruction\n");
    }
    else
    {
      info.type = InstrType::NORMAL;
      DebugLog("[InstrLogger] Normal instruction\n");
    }

    // Get destination address for CALL and JMP
    if (info.type == InstrType::CALL || info.type == InstrType::JMP)
    {
      try
      {
        if (!GetDestAddress(address, info.destination))
        {
          DebugLog("[InstrLogger] Could not get destination address\n");
          info.destination = 0;
        }
        else if (info.destination != 0)
        {
          DebugLog("[InstrLogger] Got destination address: 0x%p\n", (void *)info.destination);

          // Get destination module and symbol
          info.destModule = GetModuleNameFromAddr(info.destination);
          info.destSymbol = GetLabelFromAddress(info.destination);

          DebugLog("[InstrLogger] Destination module: %s, symbol: %s\n",
                   info.destModule.c_str(), info.destSymbol.c_str());
        }
      }
      catch (const std::exception &e)
      {
        DebugLog("[InstrLogger] Exception when getting destination info: %s\n", e.what());
        info.destination = 0;
      }
      catch (...)
      {
        DebugLog("[InstrLogger] Unknown exception when getting destination info\n");
        info.destination = 0;
      }
    }

    return true;
  }
  catch (const std::exception &e)
  {
    DebugLog("[InstrLogger] Exception in GetInstructionInfo: %s\n", e.what());
    return false;
  }
  catch (...)
  {
    DebugLog("[InstrLogger] Unknown exception in GetInstructionInfo\n");
    return false;
  }
}

bool InstrLogger::ReloadSettings()
{
  DebugLog("[%s] Reloading settings from INI file\n", PLUGIN_NAME);

  std::string pluginDir = GetPluginDirectory();
  if (pluginDir.empty())
  {
    DebugLog("[%s] Failed to get plugin directory for INI file\n", PLUGIN_NAME);
    return false;
  }

  std::string iniPath = pluginDir + CUSTOM_INI_FILENAME;
  DebugLog("[%s] Checking for INI file at: %s\n", PLUGIN_NAME, iniPath.c_str());

  // Check if the file exists
  DWORD attrs = GetFileAttributesA(iniPath.c_str());
  if (attrs == INVALID_FILE_ATTRIBUTES)
  {
    DebugLog("[%s] INI file not found at: %s\n", PLUGIN_NAME, iniPath.c_str());
    return false;
  }

  DebugLog("[%s] INI file found, loading settings\n", PLUGIN_NAME);
  bool success = LoadSettings();
  if (success)
  {
    DebugLog("[%s] Settings loaded successfully\n", PLUGIN_NAME);
  }
  else
  {
    DebugLog("[%s] Failed to load settings\n", PLUGIN_NAME);
  }

  return success;
}
