#pragma once

#include <windows.h>
#include <vector>
#include <string>
#include <unordered_set>
#include <unordered_map>
#include <fstream>
#include "../pluginsdk/_plugins.h"
#include "../pluginsdk/_dbgfunctions.h"
#include "../pluginsdk/_scriptapi.h"
#include <mutex>
#include <chrono>

// Log record structure
struct LogRecord
{
  duint address;           // Address of the instruction
  std::string instruction; // Instruction text
  duint destAddress;       // Destination address (for call/jmp)
  std::string destModule;  // Destination module name
  std::string destSymbol;  // Destination symbol name (if available)
  duint threadId;          // Thread ID
  SYSTEMTIME timestamp;    // Timestamp
};

// Logging options
struct LogOptions
{
  bool logCalls = true;
  bool logJumps = true;
  bool logReturns = false;
  bool logOnlyCurrentModule = false;
  bool followInCpu = false;
  bool logToFile = false;
  std::string outputFile = "instruction_log.csv";
  std::vector<std::pair<duint, duint>> moduleRanges; // [start, end) address ranges
  std::vector<std::pair<duint, duint>> customRanges; // Custom address ranges
  int traceStepCount = 1000000;                      // Maximum number of steps for tracing (min: 1, max: 100,000,000)

  // Constructor with default values
  LogOptions() : logCalls(true), logJumps(true), logReturns(true),
                 logOnlyCurrentModule(true), followInCpu(true),
                 logToFile(false), outputFile(""), traceStepCount(1000000) {}
};

// Instruction types
enum class InstrType
{
  UNKNOWN,
  NORMAL,
  CALL,
  JMP,
  RET
};

// Structure to store instruction information
struct InstrInfo
{
  InstrType type = InstrType::UNKNOWN;
  duint address = 0;
  duint threadId = 0;
  std::string module;
  std::string instruction;
  duint destination = 0;
  std::string destModule;
  std::string destSymbol;
  std::chrono::milliseconds timeStamp;
};

// Instruction Logger class
class InstrLogger
{
public:
  InstrLogger();
  ~InstrLogger();

  // Initialize the logger
  bool Initialize();

  // Shutdown the logger
  void Shutdown();

  // Event handlers
  void OnBreakpoint(PLUG_CB_BREAKPOINT *info);
  void OnStepped(PLUG_CB_STEPPED *info);
  void OnTraceExecute(PLUG_CB_TRACEEXECUTE *info);

  // Start/stop logging
  bool StartLogging();
  bool StopLogging();
  bool IsLogging() const;

  // Configure logging
  void SetOptions(const LogOptions &options);
  const LogOptions &GetOptions() const;

  // Add an instruction to the log
  void AddLogEntry(duint address);

  // Log instruction directly
  bool LogInstruction(duint address, duint threadId);

  // Process instruction information
  bool GetInstructionInfo(duint address, InstrInfo &info);

  // Clear log entries
  void ClearLog();

  // Get log entries
  const std::vector<LogRecord> &GetLogEntries() const;

  // Save log to file
  bool SaveLog(const std::string &filePath = "");

  // Save/load settings
  bool SaveSettings();
  bool LoadSettings();
  bool ReloadSettings(); // Reload settings from INI file

private:
  // Check if an address is within the logging range
  bool IsAddressInRange(duint address) const;

  // Format instruction for display
  std::string FormatInstruction(duint address, const std::string &instruction,
                                duint destAddress, const std::string &destModule,
                                const std::string &destSymbol);

  // Helper functions to get label from address
  std::string GetLabelFromAddress(duint addr);

  // Log storage
  std::vector<LogRecord> m_logEntries;

  // Stream for file output
  std::ofstream m_logFile;

  // Logging state
  bool m_isLogging;

  // Logging configuration
  LogOptions m_options;

  // Cached module information (mutable for const methods)
  mutable std::unordered_map<duint, bool> m_moduleCache;

  // Thread-safe instruction log
  std::vector<InstrInfo> m_instrLog;
  std::mutex m_instrMutex;
};

// Define GUI constants if not already defined
#ifndef GUI_DISASM_CENTERED
#define GUI_DISASM_CENTERED true
#endif

// Global logger instance
extern InstrLogger *g_instrLogger;

// Debug log function declaration
void DebugLog(const char *format, ...);
