#include "plugin.h"
#include "instr_logger.h"
#include <string>
#include <Windows.h>
#include <commdlg.h>
#include <commctrl.h> // Для поддержки Tooltips
#include <algorithm>  // For min and max functions
#include <Shellapi.h> // For ShellExecuteA
#include "../pluginsdk/_scriptapi_register.h"
#include "../pluginsdk/bridgemain.h" // For GUI functions
#include "../pluginsdk/_dbgfunctions.h"
// Include the ImGui settings dialog header
#include "imgui_settings_dialog.h"

// Global variables
static int g_hPlugin = 0;
static int g_hMenu = 0;
static int g_hMenuInstrLogger = 0;  // Added variable to track the submenu handle
static int g_hSettingsDialog = 0;
static HWND g_hWndParent = NULL; // Parent window handle for settings dialog

// External DebugLog function
extern void DebugLog(const char *format, ...);

// Dialog and control IDs
#define IDD_SETTINGS 1000
#define IDC_CALLS_CHECK 1001
#define IDC_JUMPS_CHECK 1002
#define IDC_RETURNS_CHECK 1003
#define IDC_MODULE_CHECK 1004
#define IDC_FOLLOW_CHECK 1005
#define IDC_FILE_CHECK 1006
#define IDC_FILE_PATH 1007
#define IDC_BROWSE_BTN 1008
#define IDC_OK_BTN 1009
#define IDC_CANCEL_BTN 1010
#define IDC_TRACE_STEPS_LABEL 1011
#define IDC_TRACE_STEPS_EDIT 1012
#define IDC_TRACE_STEPS_SPIN 1013

// Menu item constants
#define MENU_START_LOGGING 1
#define MENU_STOP_LOGGING 2
#define MENU_CLEAR_LOG 3
#define MENU_SAVE_LOG 4
#define MENU_SETTINGS 5
#define MENU_HELP 6
#define MENU_OPEN_LOG_FOLDER 7  // New menu item for opening log folder

// Forward declarations
void ShowSettingsDialog(HWND hWndParent);
void ShowSettingsGuiCallback();

// GUI callback function for showing settings dialog
void ShowSettingsGuiCallback()
{
    HWND hwnd = GuiGetWindowHandle();
    ShowSettingsDialog(hwnd);
}

// Show settings dialog using ImGui
void ShowSettingsDialog(HWND hWndParent)
{
    if (!g_instrLogger)
    {
        _plugin_logprintf("[%s] ERROR: Cannot show settings dialog - logger not initialized\n", PLUGIN_NAME);
        return;
    }

    DebugLog("[%s] Showing settings dialog\n", PLUGIN_NAME);

    // Use the ImGui-based settings dialog
    ShowImGuiSettingsDialog(hWndParent);
}

// Check if tracing functionality is available and safe to use
static bool IsTracingAvailable()
{
    // Make sure we have a valid debuggee
    if (!DbgIsDebugging())
    {
        DebugLog("[%s] IsTracingAvailable: Not debugging\n", PLUGIN_NAME);
        return false;
    }

    // Make sure the debugger is not running (must be paused to trace)
    if (DbgIsRunning())
    {
        DebugLog("[%s] IsTracingAvailable: Debugger is running, not paused\n", PLUGIN_NAME);
        return false;
    }

    // Check if we can get the CIP register - if this fails, debugger isn't ready
    duint cip = 0;
    try
    {
        cip = Script::Register::GetCIP();
        if (cip == 0)
        {
            DebugLog("[%s] IsTracingAvailable: CIP is zero\n", PLUGIN_NAME);
            return false;
        }

        // Проверка, что текущая позиция содержит корректную инструкцию
        char disasm[256] = "";
        if (!GuiGetDisassembly(cip, disasm) || disasm[0] == '\0')
        {
            DebugLog("[%s] IsTracingAvailable: Cannot disassemble at current position (CIP=%p)\n", PLUGIN_NAME, (void *)cip);
            return false;
        }
    }
    catch (...)
    {
        DebugLog("[%s] IsTracingAvailable: Exception getting CIP\n", PLUGIN_NAME);
        return false;
    }

    // Everything seems OK
    DebugLog("[%s] IsTracingAvailable: Tracing is available\n", PLUGIN_NAME);
    return true;
}

// Plugin initialization
bool pluginit(PLUG_INITSTRUCT *initStruct)
{
    initStruct->pluginVersion = PLUGIN_VERSION;
    initStruct->sdkVersion = PLUG_SDKVERSION;
    strncpy_s(initStruct->pluginName, PLUGIN_NAME, _TRUNCATE);

    // Save plugin handle
    g_hPlugin = initStruct->pluginHandle;

    DebugLog("[%s] Initializing plugin (handle: %d)\n", PLUGIN_NAME, g_hPlugin);

    // Initialize Common Controls for tooltips
    INITCOMMONCONTROLSEX icex;
    icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
    icex.dwICC = ICC_TAB_CLASSES | ICC_TREEVIEW_CLASSES | ICC_BAR_CLASSES;
    InitCommonControlsEx(&icex);
    DebugLog("[%s] Common Controls initialized\n", PLUGIN_NAME);

    // Register callbacks
    _plugin_registercallback(g_hPlugin, CB_BREAKPOINT, (CBPLUGIN)CBBREAKPOINT);
    _plugin_registercallback(g_hPlugin, CB_STEPPED, (CBPLUGIN)CBSTEPPED);
    _plugin_registercallback(g_hPlugin, CB_TRACEEXECUTE, (CBPLUGIN)CBTRACEEXECUTE);
    _plugin_registercallback(g_hPlugin, CB_MENUENTRY, (CBPLUGIN)CBMENUENTRY);
    _plugin_registercallback(g_hPlugin, CB_WINEVENT, (CBPLUGIN)CBWINEVENT);
    DebugLog("[%s] Callbacks registered\n", PLUGIN_NAME);

    // Create instruction logger
    g_instrLogger = new InstrLogger();
    if (!g_instrLogger->Initialize())
    {
        DebugLog("[%s] Failed to initialize instruction logger\n", PLUGIN_NAME);
        return false;
    }

    DebugLog("[%s] Plugin initialized successfully\n", PLUGIN_NAME);
    return true;
}

// Plugin cleanup
void plugstop()
{
    DebugLog("[%s] Plugin stopping\n", PLUGIN_NAME);

    // Clean up resources
    if (g_instrLogger)
    {
        delete g_instrLogger;
        g_instrLogger = nullptr;
        DebugLog("[%s] Instruction logger deleted\n", PLUGIN_NAME);
    }

    DebugLog("[%s] Plugin stopped\n", PLUGIN_NAME);
}

// Plugin setup
void plugsetup(PLUG_SETUPSTRUCT *setupStruct)
{
    DebugLog("[%s] Plugin setup starting\n", PLUGIN_NAME);

    // Add menu items
    g_hMenu = setupStruct->hMenu;

    // Create plugin menu
    g_hMenuInstrLogger = _plugin_menuadd(g_hMenu, "&Instruction Logger");
    DebugLog("[%s] Added main menu (handle: %d)\n", PLUGIN_NAME, g_hMenuInstrLogger);

    // Add menu entries - Logging operations
    _plugin_menuaddentry(g_hMenuInstrLogger, MENU_START_LOGGING, "▶ &Start Logging");
    _plugin_menuaddentry(g_hMenuInstrLogger, MENU_STOP_LOGGING, "■ S&top Logging");
    _plugin_menuaddseparator(g_hMenuInstrLogger);

    _plugin_menuaddentry(g_hMenuInstrLogger, MENU_CLEAR_LOG, "🗑️ &Clear Log");
    _plugin_menuaddentry(g_hMenuInstrLogger, MENU_SAVE_LOG, "💾 &Save Log...");
    _plugin_menuaddentry(g_hMenuInstrLogger, MENU_OPEN_LOG_FOLDER, "📁 Open Log &Folder");

    // Add separator between operations and configuration
    _plugin_menuaddseparator(g_hMenuInstrLogger);

    // Add configuration entries
    _plugin_menuaddentry(g_hMenuInstrLogger, MENU_SETTINGS, "⚙️ Se&ttings...");

    // Add separator before help
    _plugin_menuaddseparator(g_hMenuInstrLogger);

    // Add help entry
    _plugin_menuaddentry(g_hMenuInstrLogger, MENU_HELP, "❓ &Help");

    DebugLog("[%s] Added menu entries with user-friendly symbols\n", PLUGIN_NAME);

    // Disable stop menu initially
    _plugin_menuentrysetvisible(g_hMenuInstrLogger, MENU_STOP_LOGGING, false);

    // Display settings location
    if (g_instrLogger)
    {
        // Get plugin directory for INI file
        char pluginPath[MAX_PATH] = "";
        std::string iniPath = "Unknown Location";

        if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
            GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
        {
            char *lastSlash = strrchr(pluginPath, '\\');
            if (lastSlash)
            {
                *(lastSlash + 1) = 0; // Truncate to directory
                iniPath = std::string(pluginPath) + "InstrLoggerSettings.ini";
            }
        }

        _plugin_logprintf("[%s] Settings stored in: %s\n", PLUGIN_NAME, iniPath.c_str());

        // If settings were loaded, display them in the log
        const LogOptions &options = g_instrLogger->GetOptions();
        _plugin_logprintf("[%s] Current settings (loaded from INI file):\n", PLUGIN_NAME);
        _plugin_logprintf("  Log CALL instructions: %s\n", options.logCalls ? "Yes" : "No");
        _plugin_logprintf("  Log JMP instructions: %s\n", options.logJumps ? "Yes" : "No");
        _plugin_logprintf("  Log RET instructions: %s\n", options.logReturns ? "Yes" : "No");
        _plugin_logprintf("  Log only current module: %s\n", options.logOnlyCurrentModule ? "Yes" : "No");
        _plugin_logprintf("  Follow in CPU: %s\n", options.followInCpu ? "Yes" : "No");
        _plugin_logprintf("  Log to file: %s\n", options.logToFile ? "Yes" : "No");
        if (options.logToFile)
            _plugin_logprintf("  Output file: %s\n", options.outputFile.c_str());
    }

    DebugLog("[%s] Plugin loaded successfully!\n", PLUGIN_NAME);
    _plugin_logprintf("[%s] Plugin loaded successfully!\n", PLUGIN_NAME);
    _plugin_logprintf("[%s] Use the 'Plugins -> Instruction Logger' menu to log instructions.\n", PLUGIN_NAME);
}

// Breakpoint callback
void CBBREAKPOINT(CBTYPE cbType, PLUG_CB_BREAKPOINT *info)
{
    try
    {
        if (g_instrLogger)
            g_instrLogger->OnBreakpoint(info);
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBBREAKPOINT: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBBREAKPOINT\n", PLUGIN_NAME);
    }
}

// Step callback
void CBSTEPPED(CBTYPE cbType, PLUG_CB_STEPPED *info)
{
    try
    {
        if (g_instrLogger)
            g_instrLogger->OnStepped(info);
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBSTEPPED: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBSTEPPED\n", PLUGIN_NAME);
    }
}

// Trace execute callback
void CBTRACEEXECUTE(CBTYPE cbType, PLUG_CB_TRACEEXECUTE *info)
{
    try
    {
        if (g_instrLogger)
            g_instrLogger->OnTraceExecute(info);
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBTRACEEXECUTE: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBTRACEEXECUTE\n", PLUGIN_NAME);
    }
}

// Menu callback
void CBMENUENTRY(CBTYPE cbType, PLUG_CB_MENUENTRY *info)
{
    if (!g_instrLogger)
        return;

    try
    {
        switch (info->hEntry)
        {
        case MENU_START_LOGGING:
            DebugLog("[%s] Menu: Start Logging selected\n", PLUGIN_NAME);

            // First check if debugging is active
            if (!DbgIsDebugging())
            {
                DebugLog("[%s] Cannot start logging: No debugging session active\n", PLUGIN_NAME);
                _plugin_logprintf("[%s] Cannot start logging: No debugging session active. Please start debugging first.\n", PLUGIN_NAME);
                MessageBoxA(GuiGetWindowHandle(), "Please start a debugging session first.", PLUGIN_NAME, MB_OK | MB_ICONINFORMATION);
                break;
            }

            if (g_instrLogger->StartLogging())
            {
                // Update menu visibility - hide Start, show Stop (using the submenu handle)
                _plugin_menuentrysetvisible(g_hMenuInstrLogger, MENU_START_LOGGING, false);
                _plugin_menuentrysetvisible(g_hMenuInstrLogger, MENU_STOP_LOGGING, true);
                DebugLog("[%s] Menu visibility updated for Start/Stop\n", PLUGIN_NAME);

                // Show trace dialog to set up trace
                GuiExecuteOnGuiThread([]()
                                      {
                    try {
                        DebugLog("[%s] Setting up trace commands\n", PLUGIN_NAME);

                        // Check if tracing is available before proceeding
                        if (!IsTracingAvailable()) {
                            DebugLog("[%s] Tracing is not available at this time\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Tracing is not available. Make sure you are debugging an application.\n", PLUGIN_NAME);
                            return;
                        }

                        // Дополнительная защитная проверка перед вызовом функций трассировки
                        if (!DbgIsDebugging() || DbgIsRunning()) {
                            DebugLog("[%s] Cannot trace: debugger is not in correct state\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Cannot start trace: debugger must be debugging and paused.\n", PLUGIN_NAME);
                            return;
                        }

                        // Используем более безопасный подход без вызова потенциально опасных функций
                        try {
                            // Настраиваем файл лога через GUI функцию
                            char pluginPath[MAX_PATH] = "";
                            char fullPath[MAX_PATH] = ""; // Define fullPath variable here

                            if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                                GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH)) {

                                char* lastSlash = strrchr(pluginPath, '\\');
                                if (lastSlash) {
                                    *(lastSlash + 1) = 0; // Truncate to directory
#ifdef _WIN64
                                    sprintf_s(fullPath, "%sinstrlog_trace.trace64", pluginPath);
#else
                                    sprintf_s(fullPath, "%sinstrlog_trace.trace32", pluginPath);
#endif
                                    DebugLog("[%s] Using trace log path: %s\n", PLUGIN_NAME, fullPath);

                                    // Используем GUI функцию
                                    GuiOpenTraceFile(fullPath);
                                    DebugLog("[%s] Set trace log file via GUI function\n", PLUGIN_NAME);
                                }
                            } else {
                                // Запасной вариант
#ifdef _WIN64
                                strcpy_s(fullPath, ".\\instrlog_trace.trace64"); // Set default path
                                GuiOpenTraceFile(fullPath);
#else
                                strcpy_s(fullPath, ".\\instrlog_trace.trace32"); // Set default path
                                GuiOpenTraceFile(fullPath);
#endif
                                DebugLog("[%s] Set default trace log file\n", PLUGIN_NAME);
                            }

                            // Открываем и обновляем окно трассировки
                            GuiShowTrace();
                            Sleep(200);
                            GuiUpdateTraceBrowser();

                            // First, set up the trace log file
                            char traceLogFileCmd[MAX_PATH + 32];
                            sprintf_s(traceLogFileCmd, "TraceSetLogFile,\"%s\"", fullPath);
                            if (!DbgCmdExec(traceLogFileCmd)) {
                                DebugLog("[%s] Failed to set trace log file\n", PLUGIN_NAME);
                                return;
                            }

                            // Configure trace logging based on plugin options
                            const LogOptions &options = g_instrLogger->GetOptions();
                            std::string condition = ""; // Define condition at this scope level

                            if (options.logCalls || options.logJumps || options.logReturns) {
                                // Build the condition expression
                                bool first = true;

                                if (options.logCalls) {
                                    condition += "disasm.iscall(cip)";
                                    first = false;
                                }

                                if (options.logJumps) {
                                    if (!first) condition += " || ";
                                    condition += "disasm.isbranch(cip)";
                                    first = false;
                                }

                                if (options.logReturns) {
                                    if (!first) condition += " || ";
                                    condition += "disasm.isret(cip)";
                                    first = false;
                                }

                                // Add module restriction if needed
                                if (options.logOnlyCurrentModule) {
                                    duint modBase = 0, modSize = 0;
                                    char modName[MAX_PATH] = "";

                                    if (DbgGetModuleAt(Script::Register::GetCIP(), modName)) {
                                        // Get module base address
                                        modBase = DbgFunctions()->ModBaseFromName(modName);

                                        // Get module size if we have a valid base address
                                        if (modBase != 0) {
                                            modSize = DbgFunctions()->ModSizeFromAddr(modBase);

                                            if (modSize != 0) {
                                                char rangeCondition[128];
#ifdef _WIN64
                                                sprintf_s(rangeCondition, " && (cip >= 0x%llX && cip < 0x%llX)", modBase, modBase + modSize);
#else
                                                sprintf_s(rangeCondition, " && (cip >= 0x%X && cip < 0x%X)", modBase, modBase + modSize);
#endif
                                                condition += rangeCondition;
                                            }
                                        }
                                    }
                                }

                                // Start trace recording
                                if (!DbgCmdExec("StartTraceRecording")) {
                                    DebugLog("[%s] Failed to start trace recording\n", PLUGIN_NAME);
                                    return;
                                }

                                // Set the trace log condition (what to log)
                                char logCmd[512];
                                sprintf_s(logCmd, "TraceSetLog,\"${rip}: ${dasm}\",\"%s\"", condition.c_str());
                                if (!DbgCmdExec(logCmd)) {
                                    DebugLog("[%s] Failed to set trace log condition\n", PLUGIN_NAME);
                                    return;
                                }

                                // Теперь запускаем условную трассировку с количеством шагов из настроек
                                char traceCmd[64];
                                sprintf_s(traceCmd, "TraceIntoConditional 0,%d", options.traceStepCount);
                                if (DbgCmdExec(traceCmd)) {
                                    DebugLog("[%s] Trace recording started successfully with %d steps\n", PLUGIN_NAME, options.traceStepCount);
                                    _plugin_logprintf("[%s] Trace recording started with %d steps\n", PLUGIN_NAME, options.traceStepCount);
                                    // ... остальной код ...
                                } else {
                                    DebugLog("[%s] Failed to start conditional tracing\n", PLUGIN_NAME);
                                    _plugin_logprintf("[%s] Failed to start trace recording. Command format error or debugger not in valid state.\n", PLUGIN_NAME);

                                    // Attempt to provide more detailed error information
                                    DWORD lastError = GetLastError();
                                    if (lastError != 0) {
                                        DebugLog("[%s] System error code: %d\n", PLUGIN_NAME, lastError);
                                        _plugin_logprintf("[%s] System error code: %d\n", PLUGIN_NAME, lastError);
                                    }

                                    // Notify user that tracing failed but logging is still active
                                    _plugin_logprintf("[%s] Basic instruction logging is still active without tracing\n", PLUGIN_NAME);
                                }
                            }

                            DebugLog("[%s] Trace setup completed\n", PLUGIN_NAME);
                        }
                        catch (const std::exception& e) {
                            DebugLog("[%s] Exception in trace setup: %s\n", PLUGIN_NAME, e.what());
                            _plugin_logprintf("[%s] Error setting up trace: %s\n", PLUGIN_NAME, e.what());
                        }
                        catch (...) {
                            DebugLog("[%s] Unknown exception in trace setup\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Unknown error setting up trace\n", PLUGIN_NAME);
                        }
                    }
                    catch (const std::exception& e) {
                        DebugLog("[%s] Exception in trace setup: %s\n", PLUGIN_NAME, e.what());
                    }
                    catch (...) {
                        DebugLog("[%s] Unknown exception in trace setup\n", PLUGIN_NAME);
                    } });
            }
            break;

        case MENU_STOP_LOGGING:
            DebugLog("[%s] Menu: Stop Logging selected\n", PLUGIN_NAME);
            if (g_instrLogger->StopLogging())
            {
                // Update menu visibility - show Start, hide Stop (using the submenu handle)
                _plugin_menuentrysetvisible(g_hMenuInstrLogger, MENU_START_LOGGING, true);
                _plugin_menuentrysetvisible(g_hMenuInstrLogger, MENU_STOP_LOGGING, false);
                DebugLog("[%s] Menu visibility updated for Start/Stop\n", PLUGIN_NAME);

                // Stop tracing
                GuiExecuteOnGuiThread([]()
                                      {
                    try {
                        DebugLog("[%s] Stopping trace recording\n", PLUGIN_NAME);

                        // Автоматически останавливаем запись трассировки
                        if (DbgCmdExec("StopTraceRecording")) {  // Changed from TraceIntoStop to StopTraceRecording
                            DebugLog("[%s] Trace recording stopped automatically\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Trace recording stopped\n", PLUGIN_NAME);

                            // Автоматически сохраняем трассировку
                            try {
                                char pluginPath[MAX_PATH] = "";
                                if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                                    GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH)) {

                                    char* lastSlash = strrchr(pluginPath, '\\');
                                    if (lastSlash) {
                                        *(lastSlash + 1) = 0; // Truncate to directory
                                        char savePath[MAX_PATH] = "";

                                        // Создаем имя файла с текущей датой и временем
                                        SYSTEMTIME st;
                                        GetLocalTime(&st);

#ifdef _WIN64
                                        sprintf_s(savePath, "%sinstrlog_trace_%04d%02d%02d_%02d%02d%02d.trace64",
                                            pluginPath, st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
#else
                                        sprintf_s(savePath, "%sinstrlog_trace_%04d%02d%02d_%02d%02d%02d.trace32",
                                            pluginPath, st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
#endif

                                        // Сохраняем трассировку
                                        char cmdSave[MAX_PATH + 50] = "";
                                        sprintf_s(cmdSave, "TraceSave,\"%s\"", savePath);

                                        if (DbgCmdExec(cmdSave)) {
                                            DebugLog("[%s] Trace saved to %s\n", PLUGIN_NAME, savePath);
                                            _plugin_logprintf("[%s] Trace saved to %s\n", PLUGIN_NAME, savePath);
                                        } else {
                                            DebugLog("[%s] Failed to save trace to %s\n", PLUGIN_NAME, savePath);
                                            _plugin_logprintf("[%s] Failed to save trace. Please save manually from the trace window\n", PLUGIN_NAME);
                                        }
                                    }
                                }
                            } catch (const std::exception& e) {
                                DebugLog("[%s] Exception when saving trace: %s\n", PLUGIN_NAME, e.what());
                            } catch (...) {
                                DebugLog("[%s] Unknown exception when saving trace\n", PLUGIN_NAME);
                            }
                        }
                        else {
                            DebugLog("[%s] Failed to stop trace recording automatically\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Failed to stop trace recording\n", PLUGIN_NAME);
                        }
                    }
                    catch (const std::exception& e) {
                        DebugLog("[%s] Exception stopping trace recording: %s\n", PLUGIN_NAME, e.what());
                    }
                    catch (...) {
                        DebugLog("[%s] Unknown exception stopping trace recording\n", PLUGIN_NAME);
                    } });
            }
            break;

        case MENU_CLEAR_LOG:
            DebugLog("[%s] Menu: Clear Log selected\n", PLUGIN_NAME);
            g_instrLogger->ClearLog();
            break;

        case MENU_SAVE_LOG:
            DebugLog("[%s] Menu: Save Log selected\n", PLUGIN_NAME);
            {
                // Show file dialog
                char filename[MAX_PATH] = "";
                strncpy_s(filename, g_instrLogger->GetOptions().outputFile.c_str(), _TRUNCATE);

                OPENFILENAMEA ofn;
                ZeroMemory(&ofn, sizeof(ofn));
                ofn.lStructSize = sizeof(ofn);
                ofn.hwndOwner = GuiGetWindowHandle();
                ofn.lpstrFile = filename;
                ofn.nMaxFile = sizeof(filename);
#ifdef _WIN64
                ofn.lpstrFilter = "Trace Files (*.trace64)\0*.trace64\0CSV Files (*.csv)\0*.csv\0Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
#else
                ofn.lpstrFilter = "Trace Files (*.trace32)\0*.trace32\0CSV Files (*.csv)\0*.csv\0Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
#endif
                ofn.nFilterIndex = 1;
                ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;

                DebugLog("[%s] Showing save file dialog\n", PLUGIN_NAME);
                if (GetSaveFileNameA(&ofn))
                {
                    DebugLog("[%s] User selected save file: %s\n", PLUGIN_NAME, filename);
                    g_instrLogger->SaveLog(filename);
                }
                else
                {
                    DebugLog("[%s] Save file dialog canceled or failed (error: %d)\n", PLUGIN_NAME, CommDlgExtendedError());
                }
            }
            break;

        case MENU_SETTINGS:
            DebugLog("[%s] Menu: Settings selected\n", PLUGIN_NAME);
            if (GuiGetWindowHandle())
            {
                // First verify that the INI directory exists and is writable
                char pluginPath[MAX_PATH] = "";
                std::string iniPath = "";
                bool iniDirOK = false;

                if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                    GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
                {
                    char *lastSlash = strrchr(pluginPath, '\\');
                    if (lastSlash)
                    {
                        *(lastSlash + 1) = 0; // Truncate to directory
                        iniPath = std::string(pluginPath);

                        // Verify directory exists and is writable
                        DWORD attrs = GetFileAttributesA(iniPath.c_str());
                        if (attrs != INVALID_FILE_ATTRIBUTES && (attrs & FILE_ATTRIBUTE_DIRECTORY))
                        {
                            // Test if directory is writable
                            std::string testFile = iniPath + "test_write.tmp";
                            HANDLE hTest = CreateFileA(testFile.c_str(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
                                                       FILE_ATTRIBUTE_NORMAL, NULL);
                            if (hTest != INVALID_HANDLE_VALUE)
                            {
                                CloseHandle(hTest);
                                DeleteFileA(testFile.c_str());
                                iniDirOK = true;
                                DebugLog("[%s] INI directory verified to exist and be writable: %s\n",
                                         PLUGIN_NAME, iniPath.c_str());
                            }
                            else
                            {
                                DWORD error = GetLastError();
                                DebugLog("[%s] INI directory exists but is not writable: %s (error: %lu)\n",
                                         PLUGIN_NAME, iniPath.c_str(), error);
                            }
                        }
                        else
                        {
                            DWORD error = GetLastError();
                            DebugLog("[%s] INI directory does not exist: %s (error: %lu)\n",
                                     PLUGIN_NAME, iniPath.c_str(), error);

                            // Try to create the directory
                            if (CreateDirectoryA(iniPath.c_str(), NULL) || GetLastError() == ERROR_ALREADY_EXISTS)
                            {
                                iniDirOK = true;
                                DebugLog("[%s] Successfully created INI directory: %s\n", PLUGIN_NAME, iniPath.c_str());
                            }
                            else
                            {
                                error = GetLastError();
                                DebugLog("[%s] Failed to create INI directory: %s (error: %lu)\n",
                                         PLUGIN_NAME, iniPath.c_str(), error);
                            }
                        }
                    }
                }

                if (!iniDirOK)
                {
                    DebugLog("[%s] Problem with INI directory, displaying warning to user\n", PLUGIN_NAME);
                    MessageBoxA(GuiGetWindowHandle(),
                                "Warning: Could not access or create the directory for saving settings.\n"
                                "Settings changes may not persist between sessions.",
                                "InstrLogger Settings Warning", MB_OK | MB_ICONWARNING);
                }

                // Reload settings from INI file before showing the dialog
                if (g_instrLogger)
                {
                    DebugLog("[%s] Reloading settings from INI file before showing settings dialog\n", PLUGIN_NAME);
                    g_instrLogger->ReloadSettings();

                    // Show current settings in log
                    const LogOptions &options = g_instrLogger->GetOptions();
                    DebugLog("[%s] Displaying current settings in log\n", PLUGIN_NAME);
                    _plugin_logprintf("[%s] Current settings:\n", PLUGIN_NAME);
                    _plugin_logprintf("  Log CALL instructions: %s\n", options.logCalls ? "Yes" : "No");
                    _plugin_logprintf("  Log JMP instructions: %s\n", options.logJumps ? "Yes" : "No");
                    _plugin_logprintf("  Log RET instructions: %s\n", options.logReturns ? "Yes" : "No");
                    _plugin_logprintf("  Log only current module: %s\n", options.logOnlyCurrentModule ? "Yes" : "No");
                    _plugin_logprintf("  Follow in CPU: %s\n", options.followInCpu ? "Yes" : "No");
                    _plugin_logprintf("  Log to file: %s\n", options.logToFile ? "Yes" : "No");
                    if (options.logToFile)
                        _plugin_logprintf("  Output file: %s\n", options.outputFile.c_str());

                    // Also log INI file path information
                    _plugin_logprintf("[%s] Settings file: %s\n", PLUGIN_NAME,
                                      !iniPath.empty() ? (iniPath + CUSTOM_INI_FILENAME).c_str() : "Unknown location");
                }

                // Show the settings dialog
                g_hWndParent = GuiGetWindowHandle();
                GuiExecuteOnGuiThread(ShowSettingsGuiCallback);

                DebugLog("[%s] Settings dialog requested\n", PLUGIN_NAME);
            }
            else
            {
                // Fallback if GUI is not available
                DebugLog("[%s] Unable to get GUI window handle, showing settings in log only\n", PLUGIN_NAME);
                if (g_instrLogger)
                {
                    // Reload settings first in case they've changed
                    g_instrLogger->ReloadSettings();

                    const LogOptions &options = g_instrLogger->GetOptions();
                    _plugin_logprintf("[%s] Current settings (GUI not available):\n", PLUGIN_NAME);
                    _plugin_logprintf("  Log CALL instructions: %s\n", options.logCalls ? "Yes" : "No");
                    _plugin_logprintf("  Log JMP instructions: %s\n", options.logJumps ? "Yes" : "No");
                    _plugin_logprintf("  Log RET instructions: %s\n", options.logReturns ? "Yes" : "No");
                    _plugin_logprintf("  Log only current module: %s\n", options.logOnlyCurrentModule ? "Yes" : "No");
                    _plugin_logprintf("  Follow in CPU: %s\n", options.followInCpu ? "Yes" : "No");
                    _plugin_logprintf("  Log to file: %s\n", options.logToFile ? "Yes" : "No");
                    if (options.logToFile)
                        _plugin_logprintf("  Output file: %s\n", options.outputFile.c_str());

                    // Get plugin path to show INI location
                    char pluginPath[MAX_PATH] = "";
                    std::string iniPath = "Unknown Location";

                    if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                        GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
                    {
                        char *lastSlash = strrchr(pluginPath, '\\');
                        if (lastSlash)
                        {
                            *(lastSlash + 1) = 0; // Truncate to directory
                            iniPath = std::string(pluginPath) + CUSTOM_INI_FILENAME;
                        }
                    }

                    _plugin_logprintf("[%s] Settings file: %s\n", PLUGIN_NAME, iniPath.c_str());
                    _plugin_logprintf("[%s] To edit settings, restart x64dbg with GUI support\n", PLUGIN_NAME);
                }
            }
            break;

        case MENU_HELP:
            DebugLog("[%s] Menu: Help selected\n", PLUGIN_NAME);

            // Show brief help in log
            _plugin_logprintf("\n[%s] Showing help window...\n", PLUGIN_NAME);

            // Show the ImGui help window
            g_hWndParent = GuiGetWindowHandle();
            GuiExecuteOnGuiThread([]() {
                ShowImGuiHelpWindow(g_hWndParent);
            });

            DebugLog("[%s] Help window requested\n", PLUGIN_NAME);
            break;

        case MENU_OPEN_LOG_FOLDER:
            DebugLog("[%s] Menu: Open Log Folder selected\n", PLUGIN_NAME);
            {
                // Get the log folder path
                char logFolderPath[MAX_PATH] = "";
                bool foundPath = false;

                // First try to get the plugin directory
                if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), logFolderPath, MAX_PATH) ||
                    GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), logFolderPath, MAX_PATH))
                {
                    char* lastSlash = strrchr(logFolderPath, '\\');
                    if (lastSlash)
                    {
                        *(lastSlash + 1) = 0; // Truncate to directory
                        foundPath = true;
                    }
                }

                if (!foundPath)
                {
                    // Fallback to current directory
                    GetCurrentDirectoryA(MAX_PATH, logFolderPath);
                    foundPath = true;
                }

                if (foundPath)
                {
                    DebugLog("[%s] Opening log folder: %s\n", PLUGIN_NAME, logFolderPath);

                    // Use ShellExecute to open the folder in Explorer
                    HINSTANCE result = ShellExecuteA(NULL, "open", logFolderPath, NULL, NULL, SW_SHOWNORMAL);

                    if ((INT_PTR)result <= 32) // ShellExecute returns a value <= 32 on error
                    {
                        DebugLog("[%s] Failed to open log folder: %s (error: %d)\n", PLUGIN_NAME, logFolderPath, (int)(INT_PTR)result);
                        _plugin_logprintf("[%s] Failed to open log folder: %s\n", PLUGIN_NAME, logFolderPath);
                        MessageBoxA(GuiGetWindowHandle(), "Failed to open log folder.", PLUGIN_NAME, MB_OK | MB_ICONERROR);
                    }
                    else
                    {
                        _plugin_logprintf("[%s] Opened log folder: %s\n", PLUGIN_NAME, logFolderPath);
                    }
                }
                else
                {
                    DebugLog("[%s] Failed to determine log folder path\n", PLUGIN_NAME);
                    _plugin_logprintf("[%s] Failed to determine log folder path\n", PLUGIN_NAME);
                    MessageBoxA(GuiGetWindowHandle(), "Failed to determine log folder path.", PLUGIN_NAME, MB_OK | MB_ICONERROR);
                }
            }
            break;
        }
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBMENUENTRY: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBMENUENTRY\n", PLUGIN_NAME);
    }
}

// Window event callback
void CBWINEVENT(CBTYPE cbType, PLUG_CB_WINEVENT *info)
{
    try
    {
        // Handle window events if needed
        // Currently not used, but keeping for potential future use
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBWINEVENT: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBWINEVENT\n", PLUGIN_NAME);
    }
}

// DLL entry point
BOOL APIENTRY DllMain(HMODULE hModule, DWORD reason, LPVOID reserved)
{
    switch (reason)
    {
    case DLL_PROCESS_ATTACH:
        // Don't log here as the logger isn't initialized yet
        break;
    case DLL_PROCESS_DETACH:
        // Don't log here as the logger may be destroyed already
        break;
    case DLL_THREAD_ATTACH:
        break;
    case DLL_THREAD_DETACH:
        break;
    }
    return TRUE;
}
